@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -115,6 +116,11 @@ namespace Orthanc
     }
   }
 
+  bool HierarchicalZipWriter::Index::IsRoot() const
+  {
+    return stack_.size() == 1;
+  }
+
   std::string HierarchicalZipWriter::Index::OpenFile(const char* name)
   {
     return GetCurrentDirectoryPath() + EnsureUniqueFilename(name);
@@ -148,11 +154,50 @@ namespace Orthanc
     writer_.Open();
   }
 
+  
+  HierarchicalZipWriter::HierarchicalZipWriter(ZipWriter::IOutputStream* stream,
+                                               bool isZip64)
+  {
+    writer_.AcquireOutputStream(stream, isZip64);
+    writer_.Open();    
+  }
+
+
   HierarchicalZipWriter::~HierarchicalZipWriter()
   {
     writer_.Close();
   }
 
+  void HierarchicalZipWriter::SetZip64(bool isZip64)
+  {
+    writer_.SetZip64(isZip64);
+  }
+
+  bool HierarchicalZipWriter::IsZip64() const
+  {
+    return writer_.IsZip64();
+  }
+
+  void HierarchicalZipWriter::SetCompressionLevel(uint8_t level)
+  {
+    writer_.SetCompressionLevel(level);
+  }
+
+  uint8_t HierarchicalZipWriter::GetCompressionLevel() const
+  {
+    return writer_.GetCompressionLevel();
+  }
+
+  void HierarchicalZipWriter::SetAppendToExisting(bool append)
+  {
+    writer_.SetAppendToExisting(append);
+  }
+
+  bool HierarchicalZipWriter::IsAppendToExisting() const
+  {
+    return writer_.IsAppendToExisting();
+  }
+
   void HierarchicalZipWriter::OpenFile(const char* name)
   {
     std::string p = indexer_.OpenFile(name);
@@ -168,4 +213,40 @@ namespace Orthanc
   {
     indexer_.CloseDirectory();
   }
+
+  std::string HierarchicalZipWriter::GetCurrentDirectoryPath() const
+  {
+    return indexer_.GetCurrentDirectoryPath();
+  }
+
+  void HierarchicalZipWriter::Write(const void *data, size_t length)
+  {
+    writer_.Write(data, length);
+  }
+
+  void HierarchicalZipWriter::Write(const std::string &data)
+  {
+    writer_.Write(data);
+  }
+
+  HierarchicalZipWriter* HierarchicalZipWriter::CreateToMemory(std::string& target,
+                                                               bool isZip64)
+  {
+    return new HierarchicalZipWriter(new ZipWriter::MemoryStream(target), isZip64);
+  }
+
+  void HierarchicalZipWriter::CancelStream()
+  {
+    writer_.CancelStream();
+  }
+
+  void HierarchicalZipWriter::Close()
+  {
+    writer_.Close();
+  }
+
+  uint64_t HierarchicalZipWriter::GetArchiveSize() const
+  {
+    return writer_.GetArchiveSize();
+  }
 }
