@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -41,15 +42,9 @@ namespace Orthanc
 
     ~DicomArray();
 
-    size_t GetSize() const
-    {
-      return elements_.size();
-    }
+    size_t GetSize() const;
 
-    const DicomElement& GetElement(size_t i) const
-    {
-      return *elements_[i];
-    }
+    const DicomElement& GetElement(size_t i) const;
 
     void Print(FILE* fp) const;  // For debugging only
   };
