@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2021 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -58,10 +59,13 @@ namespace Orthanc
       virtual bool VisitDatasetTag(const DicomTag& tag,
                                    const ValueRepresentation& vr,
                                    const std::string& value,
-                                   bool isLittleEndian) = 0;
+                                   bool isLittleEndian,
+                                   uint64_t fileOffset) = 0;
     };
     
   private:
+    class PixelDataVisitor;
+    
     enum State
     {
       State_Preamble,
@@ -79,6 +83,7 @@ namespace Orthanc
     DicomTransferSyntax  transferSyntax_;
     DicomTag             danglingTag_;  // Current root-level tag
     ValueRepresentation  danglingVR_;
+    uint64_t             danglingOffset_;
     unsigned int         sequenceDepth_;
     
     bool IsLittleEndian() const;
@@ -94,7 +99,8 @@ namespace Orthanc
 
     void HandleDatasetExplicitLength(uint32_t length);
     
-    void HandleDatasetExplicitLength(const std::string& block);
+    void HandleDatasetExplicitLength(IVisitor& visitor,
+                                     const std::string& block);
 
     void HandleSequenceExplicitLength(const std::string& block);
 
@@ -121,5 +127,12 @@ namespace Orthanc
     bool IsDone() const;
 
     uint64_t GetProcessedBytes() const;
+
+    static bool LookupPixelDataOffset(uint64_t& offset,
+                                      const std::string& dicom);
+
+    static bool LookupPixelDataOffset(uint64_t& offset,
+                                      const void* buffer,
+                                      size_t size);
   };
 }
