@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2021 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -41,6 +42,7 @@ static std::string   defaultOwnPrivateKeyPath_;
 static std::string   defaultOwnCertificatePath_;
 static std::string   defaultTrustedCertificatesPath_;
 static unsigned int  defaultMaximumPduLength_ = ASC_DEFAULTMAXPDU;
+static bool          defaultRemoteCertificateRequired_ = true;
 
 
 namespace Orthanc
@@ -70,6 +72,7 @@ namespace Orthanc
     ownCertificatePath_ = defaultOwnCertificatePath_;
     trustedCertificatesPath_ = defaultTrustedCertificatesPath_;
     maximumPduLength_ = defaultMaximumPduLength_;
+    remoteCertificateRequired_ = defaultRemoteCertificateRequired_;
   }
 
 
@@ -78,8 +81,8 @@ namespace Orthanc
     timeout_(0),  // Will be set by SetDefaultParameters()
     maximumPduLength_(0)  // Will be set by SetDefaultParameters()
   {
-    remote_.SetApplicationEntityTitle("ANY-SCP");
     SetDefaultParameters();
+    remote_.SetApplicationEntityTitle("ANY-SCP");
   }
 
     
@@ -89,8 +92,8 @@ namespace Orthanc
     timeout_(0),  // Will be set by SetDefaultParameters()
     maximumPduLength_(0)  // Will be set by SetDefaultParameters()
   {
-    SetRemoteModality(remote);
     SetDefaultParameters();
+    SetRemoteModality(remote);
   }
 
   const std::string &DicomAssociationParameters::GetLocalApplicationEntityTitle() const
@@ -113,6 +116,12 @@ namespace Orthanc
   {
     CheckHost(remote.GetHost());
     remote_ = remote;
+
+    if (remote.HasTimeout())
+    {
+      timeout_ = remote.GetTimeout();
+      assert(timeout_ != 0);
+    }
   }
 
   void DicomAssociationParameters::SetRemoteApplicationEntityTitle(const std::string &aet)
@@ -231,7 +240,17 @@ namespace Orthanc
     CheckMaximumPduLength(pdu);
     maximumPduLength_ = pdu;
   }
-    
+
+  void DicomAssociationParameters::SetRemoteCertificateRequired(bool required)
+  {
+    remoteCertificateRequired_ = required;
+  }
+
+  bool DicomAssociationParameters::IsRemoteCertificateRequired() const
+  {
+    return remoteCertificateRequired_;
+  }
+
   
 
   static const char* const LOCAL_AET = "LocalAet";
@@ -241,6 +260,7 @@ namespace Orthanc
   static const char* const OWN_CERTIFICATE = "OwnCertificate";            // New in Orthanc 1.9.0
   static const char* const TRUSTED_CERTIFICATES = "TrustedCertificates";  // New in Orthanc 1.9.0
   static const char* const MAXIMUM_PDU_LENGTH = "MaximumPduLength";       // New in Orthanc 1.9.0
+  static const char* const REMOTE_CERTIFICATE_REQUIRED = "RemoteCertificateRequired";  // New in Orthanc 1.9.3
 
   
   void DicomAssociationParameters::SerializeJob(Json::Value& target) const
@@ -255,6 +275,7 @@ namespace Orthanc
       remote_.Serialize(target[REMOTE], true /* force advanced format */);
       target[TIMEOUT] = timeout_;
       target[MAXIMUM_PDU_LENGTH] = maximumPduLength_;
+      target[REMOTE_CERTIFICATE_REQUIRED] = remoteCertificateRequired_;
 
       // Don't write the DICOM TLS parameters if they are not required
       if (ownPrivateKeyPath_.empty())
@@ -335,6 +356,11 @@ namespace Orthanc
       {
         result.trustedCertificatesPath_.clear();
       }
+
+      if (serialized.isMember(REMOTE_CERTIFICATE_REQUIRED))
+      {
+        result.remoteCertificateRequired_ = SerializationToolbox::ReadBoolean(serialized, REMOTE_CERTIFICATE_REQUIRED);
+      }
       
       return result;
     }
@@ -458,4 +484,18 @@ namespace Orthanc
     boost::mutex::scoped_lock lock(defaultConfigurationMutex_);
     return defaultMaximumPduLength_;
   }
+
+
+  void DicomAssociationParameters::SetDefaultRemoteCertificateRequired(bool required)
+  {
+    boost::mutex::scoped_lock lock(defaultConfigurationMutex_);
+    defaultRemoteCertificateRequired_ = required;
+  }
+  
+
+  bool DicomAssociationParameters::GetDefaultRemoteCertificateRequired()
+  {
+    boost::mutex::scoped_lock lock(defaultConfigurationMutex_);
+    return defaultRemoteCertificateRequired_;
+  }
 }
