@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -489,7 +490,7 @@ namespace Orthanc
       return true;   // Newly created
     }
 
-    void Read(std::string& s)
+    void Write(std::string& s)
     {
       if (!GetDicomDir().write(DICOMDIR_DEFAULT_TRANSFERSYNTAX, 
                                EET_UndefinedLength /*encodingType*/, 
@@ -574,7 +575,7 @@ namespace Orthanc
 
   void DicomDirWriter::Encode(std::string& target)
   {
-    pimpl_->Read(target);
+    pimpl_->Write(target);
   }
 
 
