@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -23,6 +24,7 @@
 #include "../PrecompiledHeaders.h"
 #include "DicomWebJsonVisitor.h"
 
+#include "../Logging.h"
 #include "../OrthancException.h"
 #include "../Toolbox.h"
 #include "FromDcmtkBridge.h"
@@ -34,12 +36,14 @@
 static const char* const KEY_ALPHABETIC = "Alphabetic";
 static const char* const KEY_IDEOGRAPHIC = "Ideographic";
 static const char* const KEY_PHONETIC = "Phonetic";
+static const char* const KEY_BULK_DATA = "BulkData";
 static const char* const KEY_BULK_DATA_URI = "BulkDataURI";
 static const char* const KEY_INLINE_BINARY = "InlineBinary";
 static const char* const KEY_SQ = "SQ";
 static const char* const KEY_TAG = "tag";
 static const char* const KEY_VALUE = "Value";
 static const char* const KEY_VR = "vr";
+static const char* const KEY_NUMBER = "number";
 
 
 namespace Orthanc
@@ -118,7 +122,7 @@ namespace Orthanc
             if (content[KEY_VALUE][j].type() == Json::objectValue)
             {
               pugi::xml_node child = node.append_child("Item");
-              child.append_attribute("number").set_value(number.c_str());
+              child.append_attribute(KEY_NUMBER).set_value(number.c_str());
               ExploreXmlDataset(child, content[KEY_VALUE][j]);
             }
           }
@@ -138,7 +142,7 @@ namespace Orthanc
                 hasPhonetic)
             {
               pugi::xml_node child = node.append_child("PersonName");
-              child.append_attribute("number").set_value(number.c_str());
+              child.append_attribute(KEY_NUMBER).set_value(number.c_str());
 
               if (hasAlphabetic)
               {
@@ -161,8 +165,8 @@ namespace Orthanc
           }
           else
           {
-            pugi::xml_node child = node.append_child("Value");
-            child.append_attribute("number").set_value(number.c_str());
+            pugi::xml_node child = node.append_child(KEY_VALUE);
+            child.append_attribute(KEY_NUMBER).set_value(number.c_str());
 
             switch (content[KEY_VALUE][j].type())
             {
@@ -191,13 +195,13 @@ namespace Orthanc
       else if (content.isMember(KEY_BULK_DATA_URI) &&
                content[KEY_BULK_DATA_URI].type() == Json::stringValue)
       {
-        pugi::xml_node child = node.append_child("BulkData");
+        pugi::xml_node child = node.append_child(KEY_BULK_DATA);
         child.append_attribute("URI").set_value(content[KEY_BULK_DATA_URI].asCString());
       }
       else if (content.isMember(KEY_INLINE_BINARY) &&
                content[KEY_INLINE_BINARY].type() == Json::stringValue)
       {
-        pugi::xml_node child = node.append_child("InlineBinary");
+        pugi::xml_node child = node.append_child(KEY_INLINE_BINARY);
         child.text() = content[KEY_INLINE_BINARY].asCString();
       }
     }
@@ -336,6 +340,27 @@ namespace Orthanc
     }
   }
 
+  DicomWebJsonVisitor::DicomWebJsonVisitor() :
+    formatter_(NULL)
+  {
+    Clear();
+  }
+
+  void DicomWebJsonVisitor::SetFormatter(DicomWebJsonVisitor::IBinaryFormatter &formatter)
+  {
+    formatter_ = &formatter;
+  }
+
+  void DicomWebJsonVisitor::Clear()
+  {
+    result_ = Json::objectValue;
+  }
+
+  const Json::Value &DicomWebJsonVisitor::GetResult() const
+  {
+    return result_;
+  }
+
 
 #if ORTHANC_ENABLE_PUGIXML == 1
   void DicomWebJsonVisitor::FormatXml(std::string& target) const
@@ -347,24 +372,40 @@ namespace Orthanc
 #endif
 
 
-  void DicomWebJsonVisitor::VisitEmptySequence(const std::vector<DicomTag>& parentTags,
-                                               const std::vector<size_t>& parentIndexes,
-                                               const DicomTag& tag)
+  ITagVisitor::Action
+  DicomWebJsonVisitor::VisitNotSupported(const std::vector<DicomTag> &parentTags,
+                                         const std::vector<size_t> &parentIndexes,
+                                         const DicomTag &tag,
+                                         ValueRepresentation vr)
   {
-    if (tag.GetElement() != 0x0000)
+    return Action_None;
+  }
+
+
+  ITagVisitor::Action
+  DicomWebJsonVisitor::VisitSequence(const std::vector<DicomTag>& parentTags,
+                                     const std::vector<size_t>& parentIndexes,
+                                     const DicomTag& tag,
+                                     size_t countItems)
+  {
+    if (countItems == 0 &&
+        tag.GetElement() != 0x0000)
     {
       Json::Value& node = CreateNode(parentTags, parentIndexes, tag);
       node[KEY_VR] = EnumerationToString(ValueRepresentation_Sequence);
     }
+
+    return Action_None;
   }
   
 
-  void DicomWebJsonVisitor::VisitBinary(const std::vector<DicomTag>& parentTags,
-                                        const std::vector<size_t>& parentIndexes,
-                                        const DicomTag& tag,
-                                        ValueRepresentation vr,
-                                        const void* data,
-                                        size_t size)
+  ITagVisitor::Action
+  DicomWebJsonVisitor::VisitBinary(const std::vector<DicomTag>& parentTags,
+                                   const std::vector<size_t>& parentIndexes,
+                                   const DicomTag& tag,
+                                   ValueRepresentation vr,
+                                   const void* data,
+                                   size_t size)
   {
     assert(vr == ValueRepresentation_OtherByte ||
            vr == ValueRepresentation_OtherDouble ||
@@ -392,36 +433,49 @@ namespace Orthanc
         Json::Value& node = CreateNode(parentTags, parentIndexes, tag);
         node[KEY_VR] = EnumerationToString(vr);
 
-        switch (mode)
+        /**
+         * The test on "size > 0" is new in Orthanc 1.9.3, and fixes
+         * issue #195 (No need for BulkDataURI when Data Element is
+         * empty): https://bugs.orthanc-server.com/show_bug.cgi?id=195
+         **/
+        if (size > 0 ||
+            tag == DICOM_TAG_PIXEL_DATA ||
+            vr == ValueRepresentation_Sequence /* new in Orthanc 1.9.4 */)
         {
-          case BinaryMode_BulkDataUri:
-            node[KEY_BULK_DATA_URI] = bulkDataUri;
-            break;
-
-          case BinaryMode_InlineBinary:
+          switch (mode)
           {
-            std::string tmp(static_cast<const char*>(data), size);
+            case BinaryMode_BulkDataUri:
+              node[KEY_BULK_DATA_URI] = bulkDataUri;
+              break;
+
+            case BinaryMode_InlineBinary:
+            {
+              std::string tmp(static_cast<const char*>(data), size);
           
-            std::string base64;
-            Toolbox::EncodeBase64(base64, tmp);
+              std::string base64;
+              Toolbox::EncodeBase64(base64, tmp);
 
-            node[KEY_INLINE_BINARY] = base64;
-            break;
+              node[KEY_INLINE_BINARY] = base64;
+              break;
+            }
+
+            default:
+              throw OrthancException(ErrorCode_ParameterOutOfRange);
           }
-
-          default:
-            throw OrthancException(ErrorCode_ParameterOutOfRange);
         }
       }
     }
+
+    return Action_None;
   }
 
 
-  void DicomWebJsonVisitor::VisitIntegers(const std::vector<DicomTag>& parentTags,
-                                          const std::vector<size_t>& parentIndexes,
-                                          const DicomTag& tag,
-                                          ValueRepresentation vr,
-                                          const std::vector<int64_t>& values)
+  ITagVisitor::Action
+  DicomWebJsonVisitor::VisitIntegers(const std::vector<DicomTag>& parentTags,
+                                     const std::vector<size_t>& parentIndexes,
+                                     const DicomTag& tag,
+                                     ValueRepresentation vr,
+                                     const std::vector<int64_t>& values)
   {
     if (tag.GetElement() != 0x0000 &&
         vr != ValueRepresentation_NotSupported)
@@ -440,13 +494,16 @@ namespace Orthanc
         node[KEY_VALUE] = content;
       }
     }
+
+    return Action_None;
   }
 
-  void DicomWebJsonVisitor::VisitDoubles(const std::vector<DicomTag>& parentTags,
-                                         const std::vector<size_t>& parentIndexes,
-                                         const DicomTag& tag,
-                                         ValueRepresentation vr,
-                                         const std::vector<double>& values)
+  ITagVisitor::Action
+  DicomWebJsonVisitor::VisitDoubles(const std::vector<DicomTag>& parentTags,
+                                    const std::vector<size_t>& parentIndexes,
+                                    const DicomTag& tag,
+                                    ValueRepresentation vr,
+                                    const std::vector<double>& values)
   {
     if (tag.GetElement() != 0x0000 &&
         vr != ValueRepresentation_NotSupported)
@@ -465,13 +522,16 @@ namespace Orthanc
         node[KEY_VALUE] = content;
       }
     }
+
+    return Action_None;
   }
 
   
-  void DicomWebJsonVisitor::VisitAttributes(const std::vector<DicomTag>& parentTags,
-                                            const std::vector<size_t>& parentIndexes,
-                                            const DicomTag& tag,
-                                            const std::vector<DicomTag>& values)
+  ITagVisitor::Action
+  DicomWebJsonVisitor::VisitAttributes(const std::vector<DicomTag>& parentTags,
+                                       const std::vector<size_t>& parentIndexes,
+                                       const DicomTag& tag,
+                                       const std::vector<DicomTag>& values)
   {
     if (tag.GetElement() != 0x0000)
     {
@@ -489,6 +549,8 @@ namespace Orthanc
         node[KEY_VALUE] = content;
       }
     }
+
+    return Action_None;
   }
 
   
@@ -561,7 +623,7 @@ namespace Orthanc
               {
                 case ValueRepresentation_PersonName:
                 {
-                  Json::Value value = Json::objectValue;
+                  Json::Value tmp = Json::objectValue;
                   if (!tokens[i].empty())
                   {
                     std::vector<std::string> components;
@@ -569,21 +631,21 @@ namespace Orthanc
 
                     if (components.size() >= 1)
                     {
-                      value[KEY_ALPHABETIC] = components[0];
+                      tmp[KEY_ALPHABETIC] = components[0];
                     }
 
                     if (components.size() >= 2)
                     {
-                      value[KEY_IDEOGRAPHIC] = components[1];
+                      tmp[KEY_IDEOGRAPHIC] = components[1];
                     }
 
                     if (components.size() >= 3)
                     {
-                      value[KEY_PHONETIC] = components[2];
+                      tmp[KEY_PHONETIC] = components[2];
                     }
                   }
                   
-                  node[KEY_VALUE].append(value);
+                  node[KEY_VALUE].append(tmp);
                   break;
                 }
                   
@@ -597,15 +659,15 @@ namespace Orthanc
                    * https://groups.google.com/d/msg/orthanc-users/T32FovWPcCE/-hKFbfRJBgAJ
                    **/
 
-                  std::string t = Orthanc::Toolbox::StripSpaces(tokens[i]);
+                  std::string t = Toolbox::StripSpaces(tokens[i]);
                   if (t.empty())
                   {
                     node[KEY_VALUE].append(Json::nullValue);
                   }
                   else
                   {
-                    int64_t value = boost::lexical_cast<int64_t>(t);
-                    node[KEY_VALUE].append(FormatInteger(value));
+                    int64_t tmp = boost::lexical_cast<int64_t>(t);
+                    node[KEY_VALUE].append(FormatInteger(tmp));
                   }
                  
                   break;
@@ -613,15 +675,15 @@ namespace Orthanc
               
                 case ValueRepresentation_DecimalString:
                 {
-                  std::string t = Orthanc::Toolbox::StripSpaces(tokens[i]);
+                  std::string t = Toolbox::StripSpaces(tokens[i]);
                   if (t.empty())
                   {
                     node[KEY_VALUE].append(Json::nullValue);
                   }
                   else
                   {
-                    double value = boost::lexical_cast<double>(t);
-                    node[KEY_VALUE].append(FormatDouble(value));
+                    double tmp = boost::lexical_cast<double>(t);
+                    node[KEY_VALUE].append(FormatDouble(tmp));
                   }
 
                   break;
