@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2021 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -25,6 +26,7 @@
 #include "ITagVisitor.h"
 #include "../DicomFormat/DicomElement.h"
 #include "../DicomFormat/DicomMap.h"
+#include "../DicomFormat/DicomPath.h"
 
 #include <dcmtk/dcmdata/dcdatset.h>
 #include <dcmtk/dcmdata/dcmetinf.h>
@@ -59,6 +61,30 @@ namespace Orthanc
 
     friend class ParsedDicomFile;
 
+  public:
+    // New in Orthanc 1.9.4
+    class ORTHANC_PUBLIC IDicomPathVisitor : public boost::noncopyable
+    {
+    private:
+      static void ApplyInternal(FromDcmtkBridge::IDicomPathVisitor& visitor,
+                                DcmItem& item,
+                                const DicomPath& pattern,
+                                const DicomPath& actualPath);
+      
+    public:
+      virtual ~IDicomPathVisitor()
+      {
+      }
+
+      virtual void Visit(DcmItem& item,
+                         const DicomPath& path) = 0;
+
+      static void Apply(IDicomPathVisitor& visitor,
+                        DcmDataset& dataset,
+                        const DicomPath& path);
+    };
+    
+
   private:
     FromDcmtkBridge();  // Pure static class
 
@@ -69,7 +95,8 @@ namespace Orthanc
                               unsigned int maxStringLength,
                               Encoding encoding,
                               bool hasCodeExtensions,
-                              const std::set<DicomTag>& ignoreTagLength);
+                              const std::set<DicomTag>& ignoreTagLength,
+                              unsigned int depth);
 
     static void ElementToJson(Json::Value& parent,
                               DcmElement& element,
@@ -78,7 +105,8 @@ namespace Orthanc
                               unsigned int maxStringLength,
                               Encoding dicomEncoding,
                               bool hasCodeExtensions,
-                              const std::set<DicomTag>& ignoreTagLength);
+                              const std::set<DicomTag>& ignoreTagLength,
+                              unsigned int depth);
 
     static void ChangeStringEncoding(DcmItem& dataset,
                                      Encoding source,
@@ -86,8 +114,22 @@ namespace Orthanc
                                      Encoding target);
 
   public:
+    /**
+     * Initialize DCMTK to use the default DICOM dictionaries (either
+     * embedded into the binaries for official releases, or using the
+     * environment variable "DCM_DICT_ENVIRONMENT_VARIABLE", or using
+     * the system-wide path to the DCMTK library for developers)
+     **/
     static void InitializeDictionary(bool loadPrivateDictionary);
 
+    /**
+     * Replace the default DICOM dictionaries by the manually-provided
+     * external dictionaries. This is needed to use DICONDE for
+     * instance. Pay attention to the fact that the current dictionary
+     * will be reinitialized (all its tags are cleared).
+     **/
+    static void LoadExternalDictionaries(const std::vector<std::string>& dictionaries);
+
     static void RegisterDictionaryTag(const DicomTag& tag,
                                       ValueRepresentation vr,
                                       const std::string& name,
@@ -145,7 +187,7 @@ namespace Orthanc
 
     static void ToJson(Json::Value& result,
                        const DicomMap& values,
-                       bool simplify);
+                       DicomToJsonFormat format);
 
     static std::string GenerateUniqueIdentifier(ResourceType level);
 
@@ -222,5 +264,24 @@ namespace Orthanc
 
     static bool LookupOrthancTransferSyntax(DicomTransferSyntax& target,
                                             DcmDataset& dicom);
+
+    static void LogMissingTagsForStore(DcmDataset& dicom);
+
+    static void RemovePath(DcmDataset& dataset,
+                           const DicomPath& path);
+
+    static void ClearPath(DcmDataset& dataset,
+                          const DicomPath& path,
+                          bool onlyIfExists);
+
+    static void ReplacePath(DcmDataset& dataset,
+                            const DicomPath& path,
+                            const DcmElement& element,
+                            DicomReplaceMode mode);
+
+    static bool LookupSequenceItem(DicomMap& target,
+                                   DcmDataset& dataset,
+                                   const DicomPath& path,
+                                   size_t sequenceIndex);
   };
 }
