@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2021 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -54,6 +55,7 @@
 
 #include "ITagVisitor.h"
 #include "../DicomFormat/DicomInstanceHasher.h"
+#include "../DicomFormat/DicomPath.h"
 #include "../Images/ImageAccessor.h"
 #include "../IDynamicObject.h"
 #include "../Toolbox.h"
@@ -111,6 +113,7 @@ namespace Orthanc
     void Apply(ITagVisitor& visitor);
     ParsedDicomFile* Clone(bool keepSopInstanceUid);
     bool LookupTransferSyntax(std::string& result);
+    bool LookupTransferSyntax(std::string& result) const;
     bool GetTagValue(std::string& value,
                      const DicomTag& tag);
 #endif
@@ -209,6 +212,8 @@ namespace Orthanc
     void EmbedImage(MimeType mime,
                     const std::string& content);
 
+    void EmbedRawPixelData(const std::string& content);
+
     Encoding DetectEncoding(bool& hasCodeExtensions) const;
 
     // WARNING: This function only sets the encoding, it will not
@@ -264,7 +269,7 @@ namespace Orthanc
                              unsigned int maxTagLength,
                              const std::set<DicomTag>& ignoreTagLength) const;
 
-    bool LookupTransferSyntax(std::string& result) const;
+    bool LookupTransferSyntax(DicomTransferSyntax& result) const;
 
     bool LookupPhotometricInterpretation(PhotometricInterpretation& result) const;
 
@@ -272,5 +277,28 @@ namespace Orthanc
 
     // Decode the given frame, using the built-in DICOM decoder of Orthanc
     ImageAccessor* DecodeFrame(unsigned int frame) const;
+
+    void ReplacePath(const DicomPath& path,
+                     const Json::Value& value,  // Assumed to be encoded with UTF-8
+                     bool decodeDataUriScheme,
+                     DicomReplaceMode mode,
+                     const std::string& privateCreator /* used only for private tags */);
+
+    void RemovePath(const DicomPath& path);
+
+    void ClearPath(const DicomPath& path,
+                   bool onlyIfExists);
+
+    bool LookupSequenceItem(DicomMap& target,
+                            const DicomPath& path,
+                            size_t sequenceIndex) const;
+
+    void GetDefaultWindowing(double& windowCenter,
+                             double& windowWidth,
+                             unsigned int frame) const;
+
+    void GetRescale(double& rescaleIntercept,
+                    double& rescaleSlope,
+                    unsigned int frame) const;
   };
 }
