@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2021 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -50,8 +51,13 @@ namespace Orthanc
       value = std::string(equal + 1, end);
     }
 
+#if 0
+    // These calls were present in Orthanc <= 1.9.7, but should not be
+    // used because mongoose/civetweb already implement URL decoding
+    // (cf. "mg_url_decode()")
     Toolbox::UrlDecode(name);
     Toolbox::UrlDecode(value);
+#endif
 
     result.push_back(std::make_pair(name, value));
   }
