@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -24,6 +25,8 @@
 
 #include "IImageWriter.h"
 
+#include "../Compatibility.h"  // For ORTHANC_OVERRIDE
+
 namespace Orthanc
 {
   // https://en.wikipedia.org/wiki/Netpbm#PAM_graphics_format
@@ -35,6 +38,6 @@ namespace Orthanc
                                        unsigned int height,
                                        unsigned int pitch,
                                        PixelFormat format,
-                                       const void* buffer);
+                                       const void* buffer) ORTHANC_OVERRIDE;
   };
 }
