@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -70,8 +71,8 @@ namespace Orthanc
       if (status_.GetProgress() > 0.01f &&
           ms > 0.01f)
       {
-        float ratio = static_cast<float>(1.0 - status_.GetProgress());
-        long long remaining = boost::math::llround(ratio * ms);
+        float progress = status_.GetProgress();
+        long long remaining = boost::math::llround(ms / progress * (1.0f - progress));
         eta_ = timestamp_ + boost::posix_time::milliseconds(remaining);
         hasEta_ = true;
       }
@@ -90,6 +91,41 @@ namespace Orthanc
   {
   }
 
+  const std::string &JobInfo::GetIdentifier() const
+  {
+    return id_;
+  }
+
+  int JobInfo::GetPriority() const
+  {
+    return priority_;
+  }
+
+  JobState JobInfo::GetState() const
+  {
+    return state_;
+  }
+
+  const boost::posix_time::ptime &JobInfo::GetInfoTime() const
+  {
+    return timestamp_;
+  }
+
+  const boost::posix_time::ptime &JobInfo::GetCreationTime() const
+  {
+    return creationTime_;
+  }
+
+  const boost::posix_time::time_duration &JobInfo::GetRuntime() const
+  {
+    return runtime_;
+  }
+
+  bool JobInfo::HasEstimatedTimeOfArrival() const
+  {
+    return hasEta_;
+  }
+
 
   bool JobInfo::HasCompletionTime() const
   {
@@ -123,6 +159,16 @@ namespace Orthanc
     }
   }
 
+  const JobStatus &JobInfo::GetStatus() const
+  {
+    return status_;
+  }
+
+  JobStatus &JobInfo::GetStatus()
+  {
+    return status_;
+  }
+
 
   void JobInfo::Format(Json::Value& target) const
   {
@@ -131,6 +177,7 @@ namespace Orthanc
     target["Priority"] = priority_;
     target["ErrorCode"] = static_cast<int>(status_.GetErrorCode());
     target["ErrorDescription"] = EnumerationToString(status_.GetErrorCode());
+    target["ErrorDetails"] = status_.GetDetails();
     target["State"] = EnumerationToString(state_);
     target["Timestamp"] = boost::posix_time::to_iso_string(timestamp_);
     target["CreationTime"] = boost::posix_time::to_iso_string(creationTime_);
