@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -25,8 +26,8 @@
 
 #include "../Logging.h"
 #include "../OrthancException.h"
+#include "../Toolbox.h"
 
-#include <json/reader.h>
 
 namespace Orthanc
 {
@@ -122,7 +123,7 @@ namespace Orthanc
   {
     assert(engine != NULL);
 
-    LOG(INFO) << "Worker thread " << workerIndex << " has started";
+    CLOG(INFO, JOBS) << "Worker thread " << workerIndex << " has started";
 
     while (engine->IsRunning())
     {
@@ -130,8 +131,8 @@ namespace Orthanc
 
       if (running.IsValid())
       {
-        LOG(INFO) << "Executing job with priority " << running.GetPriority()
-                  << " in worker thread " << workerIndex << ": " << running.GetId();
+        CLOG(INFO, JOBS) << "Executing job with priority " << running.GetPriority()
+                         << " in worker thread " << workerIndex << ": " << running.GetId();
 
         while (engine->IsRunning())
         {
@@ -159,7 +160,7 @@ namespace Orthanc
     if (state_ != State_Setup &&
         state_ != State_Done)
     {
-      LOG(ERROR) << "INTERNAL ERROR: JobsEngine::Stop() should be invoked manually to avoid mess in the destruction order!";
+      CLOG(ERROR, JOBS) << "INTERNAL ERROR: JobsEngine::Stop() should be invoked manually to avoid mess in the destruction order!";
       Stop();
     }
   }
@@ -197,8 +198,7 @@ namespace Orthanc
                                           const std::string& serialized)
   {
     Json::Value value;
-    Json::Reader reader;
-    if (reader.parse(serialized, value))
+    if (Toolbox::ReadJson(value, serialized))
     {
       LoadRegistryFromJson(unserializer, value);
     }
@@ -269,7 +269,7 @@ namespace Orthanc
 
     state_ = State_Running;
 
-    LOG(WARNING) << "The jobs engine has started with " << workers_.size() << " threads";
+    CLOG(WARNING, JOBS) << "The jobs engine has started with " << workers_.size() << " threads";
   }
 
 
@@ -286,7 +286,7 @@ namespace Orthanc
       state_ = State_Stopping;
     }
 
-    LOG(INFO) << "Stopping the jobs engine";
+    CLOG(INFO, JOBS) << "Stopping the jobs engine";
       
     if (retryHandler_.joinable())
     {
@@ -310,6 +310,6 @@ namespace Orthanc
       state_ = State_Done;
     }
 
-    LOG(WARNING) << "The jobs engine has stopped";
+    CLOG(WARNING, JOBS) << "The jobs engine has stopped";
   }
 }
