@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -39,6 +40,8 @@ namespace Orthanc
   static const char* CREATION_TIME = "CreationTime";
   static const char* LAST_CHANGE_TIME = "LastChangeTime";
   static const char* RUNTIME = "Runtime";
+  static const char* ERROR_CODE = "ErrorCode";
+  static const char* ERROR_DETAILS = "ErrorDetails";
 
 
   class JobsRegistry::JobHandler : public boost::noncopyable
@@ -224,6 +227,11 @@ namespace Orthanc
       return runtime_;
     }
 
+    void ResetRuntime()
+    {
+      runtime_ = boost::posix_time::milliseconds(0);
+    }
+
     const JobStatus& GetLastStatus() const
     {
       return lastStatus_;
@@ -276,11 +284,16 @@ namespace Orthanc
         target[CREATION_TIME] = boost::posix_time::to_iso_string(creationTime_);
         target[LAST_CHANGE_TIME] = boost::posix_time::to_iso_string(lastStateChangeTime_);
         target[RUNTIME] = static_cast<unsigned int>(runtime_.total_milliseconds());
+
+        // New in Orthanc 1.9.5
+        target[ERROR_CODE] = static_cast<int>(lastStatus_.GetErrorCode());
+        target[ERROR_DETAILS] = lastStatus_.GetDetails();
+        
         return true;
       }
       else
       {
-        VLOG(1) << "Job backup is not supported for job of type: " << jobType_;
+        LOG(TRACE) << "Job backup is not supported for job of type: " << jobType_;
         return false;
       }
     }
@@ -307,13 +320,29 @@ namespace Orthanc
       job_->GetJobType(jobType_);
       job_->Start();
 
-      lastStatus_ = JobStatus(ErrorCode_Success, "", *job_);
+      ErrorCode errorCode;
+      if (serialized.isMember(ERROR_CODE))
+      {
+        errorCode = static_cast<ErrorCode>(SerializationToolbox::ReadInteger(serialized, ERROR_CODE));
+      }
+      else
+      {
+        errorCode = ErrorCode_Success;  // Backward compatibility with Orthanc <= 1.9.4
+      }
+
+      std::string details;
+      if (serialized.isMember(ERROR_DETAILS))  // Backward compatibility with Orthanc <= 1.9.4
+      {
+        details = SerializationToolbox::ReadString(serialized, ERROR_DETAILS);
+      }
+
+      lastStatus_ = JobStatus(errorCode, details, *job_);
     }
   };
 
 
-  bool JobsRegistry::PriorityComparator::operator() (JobHandler*& a,
-                                                     JobHandler*& b) const
+  bool JobsRegistry::PriorityComparator::operator() (JobHandler* const& a,
+                                                     JobHandler* const& b) const
   {
     return a->GetPriority() < b->GetPriority();
   }
@@ -622,6 +651,7 @@ namespace Orthanc
 
   bool JobsRegistry::GetJobOutput(std::string& output,
                                   MimeType& mime,
+                                  std::string& filename,
                                   const std::string& job,
                                   const std::string& key)
   {
@@ -640,7 +670,7 @@ namespace Orthanc
 
       if (handler.GetState() == JobState_Success)
       {
-        return handler.GetJob().GetOutput(output, mime, key);
+        return handler.GetJob().GetOutput(output, mime, filename, key);
       }
       else
       {
@@ -711,6 +741,12 @@ namespace Orthanc
     }
   }
 
+  JobsRegistry::JobsRegistry(size_t maxCompletedJobs) :
+    maxCompletedJobs_(maxCompletedJobs),
+    observer_(NULL)
+  {
+  }
+
 
   void JobsRegistry::Submit(std::string& id,
                             IJob* job,        // Takes ownership
@@ -1042,6 +1078,7 @@ namespace Orthanc
       (void) ok;  // Remove warning about unused variable in release builds
       assert(ok);
 
+      found->second->ResetRuntime();
       found->second->SetState(JobState_Pending);
       pendingJobs_.push(found->second);
       pendingJobAvailable_.notify_one();
