@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -22,27 +23,19 @@
 
 #pragma once
 
-#include "JobOperationValue.h"
+#include "IJobOperationValue.h"
+
+#include "../../Compatibility.h"  // For ORTHANC_OVERRIDE
 
 namespace Orthanc
 {
-  class ORTHANC_PUBLIC NullOperationValue : public JobOperationValue
+  class ORTHANC_PUBLIC NullOperationValue : public IJobOperationValue
   {
   public:
-    NullOperationValue() :
-      JobOperationValue(Type_Null)
-    {
-    }
+    virtual Type GetType() const ORTHANC_OVERRIDE;
+    
+    virtual IJobOperationValue* Clone() const ORTHANC_OVERRIDE;
 
-    virtual JobOperationValue* Clone() const
-    {
-      return new NullOperationValue;
-    }
-
-    virtual void Serialize(Json::Value& target) const
-    {
-      target = Json::objectValue;
-      target["Type"] = "Null";
-    }
+    virtual void Serialize(Json::Value& target) const ORTHANC_OVERRIDE;
   };
 }
