@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -24,6 +25,8 @@
 
 #include "IJob.h"
 
+#include "../Compatibility.h"  // For ORTHANC_OVERRIDE
+
 #include <set>
 
 namespace Orthanc
@@ -68,50 +71,29 @@ namespace Orthanc
 
     virtual ~SetOfCommandsJob();
 
-    size_t GetPosition() const
-    {
-      return position_;
-    }
+    size_t GetPosition() const;
 
-    void SetDescription(const std::string& description)
-    {
-      description_ = description;
-    }
+    void SetDescription(const std::string& description);
 
-    const std::string& GetDescription() const
-    {
-      return description_;
-    }
+    const std::string& GetDescription() const;
 
     void Reserve(size_t size);
 
-    size_t GetCommandsCount() const
-    {
-      return commands_.size();
-    }
+    size_t GetCommandsCount() const;
 
     void AddCommand(ICommand* command);  // Takes ownership
 
-    bool IsPermissive() const
-    {
-      return permissive_;
-    }
+    bool IsPermissive() const;
 
     void SetPermissive(bool permissive);
 
     virtual void Reset() ORTHANC_OVERRIDE;
     
-    virtual void Start() ORTHANC_OVERRIDE
-    {
-      started_ = true;
-    }
+    virtual void Start() ORTHANC_OVERRIDE;
     
     virtual float GetProgress() ORTHANC_OVERRIDE;
 
-    bool IsStarted() const
-    {
-      return started_;
-    }
+    bool IsStarted() const;
 
     const ICommand& GetCommand(size_t index) const;
       
@@ -123,9 +105,7 @@ namespace Orthanc
 
     virtual bool GetOutput(std::string& output,
                            MimeType& mime,
-                           const std::string& key) ORTHANC_OVERRIDE
-    {
-      return false;
-    }
+                           std::string& filename,
+                           const std::string& key) ORTHANC_OVERRIDE;
   };
 }
