@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -51,14 +52,11 @@ namespace Orthanc
   public:
     SetOfInstancesJob();
 
-    SetOfInstancesJob(const Json::Value& source);  // Unserialization
+    explicit SetOfInstancesJob(const Json::Value& source);  // Unserialization
 
     // Only used for reporting in the public content
     // https://groups.google.com/d/msg/orthanc-users/9GCV88GLEzw/6wAgP_PRAgAJ
-    void AddParentResource(const std::string& resource)
-    {
-      parentResources_.insert(resource);
-    }
+    void AddParentResource(const std::string& resource);
     
     void AddInstance(const std::string& instance);
 
@@ -68,27 +66,18 @@ namespace Orthanc
     
     const std::string& GetInstance(size_t index) const;
 
-    bool HasTrailingStep() const
-    {
-      return hasTrailingStep_;
-    }
+    bool HasTrailingStep() const;
 
-    const std::set<std::string>& GetFailedInstances() const
-    {
-      return failedInstances_;
-    }
+    const std::set<std::string>& GetFailedInstances() const;
 
-    bool IsFailedInstance(const std::string& instance) const
-    {
-      return failedInstances_.find(instance) != failedInstances_.end();
-    }
+    bool IsFailedInstance(const std::string& instance) const;
 
-    virtual void Start();
+    virtual void Start() ORTHANC_OVERRIDE;
 
-    virtual void Reset();
+    virtual void Reset() ORTHANC_OVERRIDE;
 
-    virtual void GetPublicContent(Json::Value& target);
+    virtual void GetPublicContent(Json::Value& target) ORTHANC_OVERRIDE;
 
-    virtual bool Serialize(Json::Value& target);
+    virtual bool Serialize(Json::Value& target) ORTHANC_OVERRIDE;
   };
 }
