@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -25,11 +26,24 @@
 
 #include "OrthancException.h"
 
+#include <stdint.h>
+
+
+/*********************************************************
+ * Common section
+ *********************************************************/
 
 namespace Orthanc
 {
   namespace Logging
   {
+    static const uint32_t ALL_CATEGORIES_MASK = 0xffffffff;
+    
+    static uint32_t infoCategoriesMask_ = 0;
+    static uint32_t traceCategoriesMask_ = 0;
+    static std::string logTargetFolder_;            // keep a track of the log folder in case of reset of the context
+    static std::string logTargetFile_;              // keep a track of the log file in case of reset of the context
+    
     const char* EnumerationToString(LogLevel level)
     {
       switch (level)
@@ -75,12 +89,219 @@ namespace Orthanc
         throw OrthancException(ErrorCode_InternalError);
       }
     }
+
+    
+    void EnableInfoLevel(bool enabled)
+    {
+      if (enabled)
+      {
+        infoCategoriesMask_ = ALL_CATEGORIES_MASK;
+      }
+      else
+      {
+        // Also disable the "TRACE" level when info-level debugging is disabled
+        infoCategoriesMask_ = 0;
+        traceCategoriesMask_ = 0;
+      }
+    }
+
+    
+    bool IsInfoLevelEnabled()
+    {
+      return (infoCategoriesMask_ != 0);
+    }
+
+    
+    void EnableTraceLevel(bool enabled)
+    {
+      if (enabled)
+      {
+        // Also enable the "INFO" level when trace-level debugging is enabled
+        infoCategoriesMask_ = ALL_CATEGORIES_MASK;
+        traceCategoriesMask_ = ALL_CATEGORIES_MASK;
+      }
+      else
+      {
+        traceCategoriesMask_ = 0;
+      }
+    }
+
+
+    bool IsTraceLevelEnabled()
+    {
+      return (traceCategoriesMask_ != 0);
+    }
+
+    
+    void SetCategoryEnabled(LogLevel level,
+                            LogCategory category,
+                            bool enabled)
+    {
+      // Invariant: If a bit is set for "trace", it must also be set
+      // for "verbose" (in other words, trace level implies verbose level)
+      assert((traceCategoriesMask_ & infoCategoriesMask_) == traceCategoriesMask_);
+      
+      if (level == LogLevel_INFO)
+      {
+        if (enabled)
+        {
+          infoCategoriesMask_ |= static_cast<uint32_t>(category);
+        }
+        else
+        {
+          infoCategoriesMask_ &= ~static_cast<uint32_t>(category);
+          traceCategoriesMask_ &= ~static_cast<uint32_t>(category);
+        }
+      }
+      else if (level == LogLevel_TRACE)
+      {
+        if (enabled)
+        {
+          traceCategoriesMask_ |= static_cast<uint32_t>(category);
+          infoCategoriesMask_ |= static_cast<uint32_t>(category);
+        }
+        else
+        {
+          traceCategoriesMask_ &= ~static_cast<uint32_t>(category);
+        }
+      }
+      else
+      {
+        throw OrthancException(ErrorCode_ParameterOutOfRange,
+                               "Can only modify the parameters of the INFO and TRACE levels");
+      }
+
+      assert((traceCategoriesMask_ & infoCategoriesMask_) == traceCategoriesMask_);
+    }
+
+    
+    bool IsCategoryEnabled(LogLevel level,
+                           LogCategory category)
+    {
+      if (level == LogLevel_ERROR ||
+          level == LogLevel_WARNING)
+      {
+        return true;
+      }
+      else if (level == LogLevel_INFO)
+      {
+        return (infoCategoriesMask_ & category) != 0;
+      }
+      else if (level == LogLevel_TRACE)
+      {
+        return (traceCategoriesMask_ & category) != 0;
+      }
+      else
+      {
+        return false;
+      }
+    }
+
+
+    bool LookupCategory(LogCategory& target,
+                        const std::string& category)
+    {
+      if (category == "generic")
+      {
+        target = LogCategory_GENERIC;
+        return true;
+      }
+      else if (category == "plugins")
+      {
+        target = LogCategory_PLUGINS;
+        return true;
+      }
+      else if (category == "http")
+      {
+        target = LogCategory_HTTP;
+        return true;
+      }
+      else if (category == "dicom")
+      {
+        target = LogCategory_DICOM;
+        return true;
+      }
+      else if (category == "sqlite")
+      {
+        target = LogCategory_SQLITE;
+        return true;
+      }
+      else if (category == "jobs")
+      {
+        target = LogCategory_JOBS;
+        return true;
+      }
+      else if (category == "lua")
+      {
+        target = LogCategory_LUA;
+        return true;
+      }
+      else
+      {
+        return false;
+      }
+    }
+
+
+    unsigned int GetCategoriesCount()
+    {
+      return 7;
+    }
+
+
+    const char* GetCategoryName(unsigned int i)
+    {
+      if (i < GetCategoriesCount())
+      {
+        return GetCategoryName(static_cast<LogCategory>(1 << i));
+      }
+      else
+      {
+        throw OrthancException(ErrorCode_ParameterOutOfRange);
+      }
+    }
+    
+
+    const char* GetCategoryName(LogCategory category)
+    {
+      switch (category)
+      {
+        case LogCategory_GENERIC:
+          return "generic";
+            
+        case LogCategory_PLUGINS:
+          return "plugins";
+            
+        case LogCategory_HTTP:
+          return "http";
+            
+        case LogCategory_DICOM:
+          return "dicom";
+            
+        case LogCategory_SQLITE:
+          return "sqlite";
+
+        case LogCategory_JOBS:
+          return "jobs";
+
+        case LogCategory_LUA:
+          return "lua";
+
+        default:
+          throw OrthancException(ErrorCode_ParameterOutOfRange);
+      }
+    }
   }
 }
 
 
+
 #if ORTHANC_ENABLE_LOGGING != 1
 
+/*********************************************************
+ * Section if logging is disabled
+ *********************************************************/
+
 namespace Orthanc
 {
   namespace Logging
@@ -105,24 +326,6 @@ namespace Orthanc
     {
     }
 
-    void EnableInfoLevel(bool enabled)
-    {
-    }
-
-    void EnableTraceLevel(bool enabled)
-    {
-    }
-    
-    bool IsTraceLevelEnabled()
-    {
-      return false;
-    }
-
-    bool IsInfoLevelEnabled()
-    {
-      return false;
-    }
-    
     void SetTargetFile(const std::string& path)
     {
     }
@@ -152,9 +355,6 @@ namespace Orthanc
 {
   namespace Logging
   {
-    static bool infoEnabled_ = false;
-    static bool traceEnabled_ = false;
-    
 #ifdef __EMSCRIPTEN__
     static void ErrorLogFunc(const char* msg)
     {
@@ -202,37 +402,34 @@ namespace Orthanc
     {
       std::string message = messageStream_.str();
 
-      switch (level_)
+      if (IsCategoryEnabled(level_, category_))
       {
-        case LogLevel_ERROR:
-          ErrorLogFunc(message.c_str());
-          break;
+        switch (level_)
+        {
+          case LogLevel_ERROR:
+            ErrorLogFunc(message.c_str());
+            break;
 
-        case LogLevel_WARNING:
-          WarningLogFunc(message.c_str());
-          break;
+          case LogLevel_WARNING:
+            WarningLogFunc(message.c_str());
+            break;
 
-        case LogLevel_INFO:
-          if (infoEnabled_)
-          {
+          case LogLevel_INFO:
             InfoLogFunc(message.c_str());
             // TODO: stone_console_info(message_.c_str());
-          }
-          break;
+            break;
 
-        case LogLevel_TRACE:
-          if (traceEnabled_)
-          {
+          case LogLevel_TRACE:
             TraceLogFunc(message.c_str());
-          }
-          break;
+            break;
 
-        default:
-        {
-          std::stringstream ss;
-          ss << "Unknown log level (" << level_ << ") for message: " << message;
-          std::string s = ss.str();
-          ErrorLogFunc(s.c_str());
+          default:
+          {
+            std::stringstream ss;
+            ss << "Unknown log level (" << level_ << ") for message: " << message;
+            std::string s = ss.str();
+            ErrorLogFunc(s.c_str());
+          }
         }
       }
     }
@@ -257,32 +454,6 @@ namespace Orthanc
     {
     }
 
-    void EnableInfoLevel(bool enabled)
-    {
-      infoEnabled_ = enabled;
-
-      if (!enabled)
-      {
-        // Also disable the "TRACE" level when info-level debugging is disabled
-        traceEnabled_ = false;
-      }
-    }
-
-    bool IsInfoLevelEnabled()
-    {
-      return infoEnabled_;
-    }
-
-    void EnableTraceLevel(bool enabled)
-    {
-      traceEnabled_ = enabled;
-    }
-
-    bool IsTraceLevelEnabled()
-    {
-      return traceEnabled_;
-    }
-
     void SetTargetFile(const std::string& path)
     {
     }
@@ -367,8 +538,6 @@ static std::unique_ptr<LoggingStreamsContext> loggingStreamsContext_;
 static boost::mutex                           loggingStreamsMutex_;
 static Orthanc::Logging::NullStream           nullStream_;
 static OrthancPluginContext*                  pluginContext_ = NULL;
-static bool                                   infoEnabled_ = false;
-static bool                                   traceEnabled_ = false;
 
 
 namespace Orthanc
@@ -448,7 +617,8 @@ namespace Orthanc
     static void GetLinePrefix(std::string& prefix,
                               LogLevel level,
                               const char* file,
-                              int line)
+                              int line,
+                              LogCategory category)
     {
       boost::filesystem::path path(file);
       boost::posix_time::ptime now = boost::posix_time::microsec_clock::local_time();
@@ -495,7 +665,8 @@ namespace Orthanc
           break;
 
         default:
-          throw OrthancException(ErrorCode_InternalError);            
+          c = '?';
+          break;
       }
 
       char date[64];
@@ -510,6 +681,13 @@ namespace Orthanc
 
       prefix = (std::string(date) + path.filename().string() + ":" +
                 boost::lexical_cast<std::string>(line) + "] ");
+
+      if (level != LogLevel_ERROR &&
+          level != LogLevel_WARNING &&
+          category != LogCategory_GENERIC)
+      {
+        prefix += "(" + std::string(GetCategoryName(category)) + ") ";
+      }
     }
     
 
@@ -541,72 +719,23 @@ namespace Orthanc
 
     void Reset()
     {
-      // Recover the old logging context
-      std::unique_ptr<LoggingStreamsContext> old;
-
       {
         boost::mutex::scoped_lock lock(loggingStreamsMutex_);
-        if (loggingStreamsContext_.get() == NULL)
-        {
-          return;
-        }
-        else
-        {
-#if __cplusplus < 201103L
-          old.reset(loggingStreamsContext_.release());
-#else
-          old = std::move(loggingStreamsContext_);
-#endif
-
-          // Create a new logging context, 
-          loggingStreamsContext_.reset(new LoggingStreamsContext);
-        }
+        loggingStreamsContext_.reset(new LoggingStreamsContext);
       }
-      
-      if (!old->targetFolder_.empty())
+
+      // Recover the old logging context if any
+      if (!logTargetFile_.empty())
       {
-        SetTargetFolder(old->targetFolder_);
+        SetTargetFile(logTargetFile_);
       }
-      else if (!old->targetFile_.empty())
+      else if (!logTargetFolder_.empty())
       {
-        SetTargetFile(old->targetFile_);
+        SetTargetFolder(logTargetFolder_);
       }
     }
 
 
-    void EnableInfoLevel(bool enabled)
-    {
-      infoEnabled_ = enabled;
-      
-      if (!enabled)
-      {
-        // Also disable the "TRACE" level when info-level debugging is disabled
-        traceEnabled_ = false;
-      }
-    }
-
-    bool IsInfoLevelEnabled()
-    {
-      return infoEnabled_;
-    }
-
-    void EnableTraceLevel(bool enabled)
-    {
-      traceEnabled_ = enabled;
-      
-      if (enabled)
-      {
-        // Also enable the "INFO" level when trace-level debugging is enabled
-        infoEnabled_ = true;
-      }
-    }
-
-    bool IsTraceLevelEnabled()
-    {
-      return traceEnabled_;
-    }
-
-
     void SetTargetFolder(const std::string& path)
     {
       boost::mutex::scoped_lock lock(loggingStreamsMutex_);
@@ -620,6 +749,7 @@ namespace Orthanc
         loggingStreamsContext_->warning_ = loggingStreamsContext_->file_.get();
         loggingStreamsContext_->error_ = loggingStreamsContext_->file_.get();
         loggingStreamsContext_->info_ = loggingStreamsContext_->file_.get();
+        logTargetFolder_ = path;
       }
     }
 
@@ -638,22 +768,21 @@ namespace Orthanc
         loggingStreamsContext_->warning_ = loggingStreamsContext_->file_.get();
         loggingStreamsContext_->error_ = loggingStreamsContext_->file_.get();
         loggingStreamsContext_->info_ = loggingStreamsContext_->file_.get();
+        logTargetFile_ = path;
       }
     }
 
 
-    InternalLogger::InternalLogger(LogLevel level,
-                                   const char* file,
-                                   int line) : 
-      lock_(loggingStreamsMutex_, boost::defer_lock_t()),
-      level_(level),
-      stream_(&nullStream_)  // By default, logging to "/dev/null" is simulated
+    void InternalLogger::Setup(LogCategory category,
+                               const char* file,
+                               int line)
     {
       if (pluginContext_ != NULL)
       {
         // We are logging using the Orthanc plugin SDK
 
-        if (level == LogLevel_TRACE)
+        if (level_ == LogLevel_TRACE ||
+            !IsCategoryEnabled(level_, category))
         {
           // No trace level in plugins, directly exit as the stream is
           // set to "/dev/null"
@@ -669,8 +798,7 @@ namespace Orthanc
       {
         // We are logging in a standalone application, not inside an Orthanc plugin
 
-        if ((level == LogLevel_INFO  && !infoEnabled_) ||
-            (level == LogLevel_TRACE && !traceEnabled_))
+        if (!IsCategoryEnabled(level_, category))
         {
           // This logging level is disabled, directly exit as the
           // stream is set to "/dev/null"
@@ -678,7 +806,7 @@ namespace Orthanc
         }
 
         std::string prefix;
-        GetLinePrefix(prefix, level, file, line);
+        GetLinePrefix(prefix, level_, file, line, category);
 
         {
           // We lock the global mutex. The mutex is locked until the
@@ -692,7 +820,7 @@ namespace Orthanc
             return;
           }
 
-          switch (level)
+          switch (level_)
           {
             case LogLevel_ERROR:
               stream_ = loggingStreamsContext_->error_;
@@ -707,8 +835,9 @@ namespace Orthanc
               stream_ = loggingStreamsContext_->info_;
               break;
               
-            default:
-              throw OrthancException(ErrorCode_InternalError);
+            default:  // Should not occur
+              stream_ = loggingStreamsContext_->error_;
+              break;              
           }
 
           if (stream_ == &nullStream_)
@@ -736,6 +865,29 @@ namespace Orthanc
     }
 
 
+    InternalLogger::InternalLogger(LogLevel level,
+                                   LogCategory category,
+                                   const char* file,
+                                   int line) :
+      lock_(loggingStreamsMutex_, boost::defer_lock_t()),
+      level_(level),
+      stream_(&nullStream_)  // By default, logging to "/dev/null" is simulated
+    {
+      Setup(category, file, line);
+    }
+
+
+    InternalLogger::InternalLogger(LogLevel level,
+                                   const char* file,
+                                   int line) :
+      lock_(loggingStreamsMutex_, boost::defer_lock_t()),
+      level_(level),
+      stream_(&nullStream_)  // By default, logging to "/dev/null" is simulated
+    {
+      Setup(LogCategory_GENERIC, file, line);
+    }
+
+
     InternalLogger::~InternalLogger()
     {
       if (pluginStream_.get() != NULL)
