@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -24,6 +25,7 @@
 
 // To have ORTHANC_ENABLE_LOGGING defined if using the shared library
 #include "OrthancFramework.h"
+#include "Compatibility.h"
 
 #include <iostream>
 
@@ -51,6 +53,23 @@ namespace Orthanc
       LogLevel_INFO,
       LogLevel_TRACE
     };
+
+    /**
+     * NB: The log level for each category is encoded as a bit
+     * mask. As a consequence, there can be up to 31 log categories
+     * (not 32, as the value GENERIC is reserved for the log commands
+     * that don't fall in a specific category).
+     **/
+    enum LogCategory
+    {
+      LogCategory_GENERIC = (1 << 0),
+      LogCategory_PLUGINS = (1 << 1),
+      LogCategory_HTTP    = (1 << 2),
+      LogCategory_SQLITE  = (1 << 3),
+      LogCategory_DICOM   = (1 << 4),
+      LogCategory_JOBS    = (1 << 5),
+      LogCategory_LUA     = (1 << 6)
+    };
     
     ORTHANC_PUBLIC const char* EnumerationToString(LogLevel level);
 
@@ -75,11 +94,27 @@ namespace Orthanc
 
     ORTHANC_PUBLIC bool IsInfoLevelEnabled();
 
+    ORTHANC_PUBLIC void SetCategoryEnabled(LogLevel level,
+                                           LogCategory category,
+                                           bool enabled);
+
+    ORTHANC_PUBLIC bool IsCategoryEnabled(LogLevel level,
+                                          LogCategory category);
+    
+    ORTHANC_PUBLIC bool LookupCategory(LogCategory& target,
+                                       const std::string& category);
+
+    ORTHANC_PUBLIC unsigned int GetCategoriesCount();
+
+    ORTHANC_PUBLIC const char* GetCategoryName(unsigned int i);
+
+    ORTHANC_PUBLIC const char* GetCategoryName(LogCategory category);
+
     ORTHANC_PUBLIC void SetTargetFile(const std::string& path);
 
     ORTHANC_PUBLIC void SetTargetFolder(const std::string& path);
 
-    struct NullStream : public std::ostream 
+    struct ORTHANC_LOCAL NullStream : public std::ostream 
     {
       NullStream() : 
         std::ios(0), 
@@ -97,14 +132,40 @@ namespace Orthanc
 }
 
 
+
+/**
+ * NB:
+ * - The "VLOG(unused)" macro is for backward compatibility with
+ *   Orthanc <= 1.8.0.
+ * - The "CLOG()" macro stands for "category log" (new in Orthanc 1.8.1)
+ **/
+
+#if defined(LOG)
+#  error The macro LOG cannot be defined beforehand
+#endif
+
+#if defined(VLOG)
+#  error The macro VLOG cannot be defined beforehand
+#endif
+
+#if defined(CLOG)
+#  error The macro CLOG cannot be defined beforehand
+#endif
+
 #if ORTHANC_ENABLE_LOGGING != 1
-#  define LOG(level)   ::Orthanc::Logging::NullStream()
-#  define VLOG(level)  ::Orthanc::Logging::NullStream()
+#  define LOG(level)            ::Orthanc::Logging::NullStream()
+#  define VLOG(unused)          ::Orthanc::Logging::NullStream()
+#  define CLOG(level, category) ::Orthanc::Logging::NullStream()
 #else /* ORTHANC_ENABLE_LOGGING == 1 */
-#  define LOG(level)  ::Orthanc::Logging::InternalLogger        \
-  (::Orthanc::Logging::LogLevel_ ## level, __FILE__, __LINE__)
-#  define VLOG(level) ::Orthanc::Logging::InternalLogger        \
-  (::Orthanc::Logging::LogLevel_TRACE, __FILE__, __LINE__)
+#  define LOG(level)     ::Orthanc::Logging::InternalLogger     \
+  (::Orthanc::Logging::LogLevel_ ## level,                      \
+   ::Orthanc::Logging::LogCategory_GENERIC, __FILE__, __LINE__)
+#  define VLOG(unused)   ::Orthanc::Logging::InternalLogger     \
+  (::Orthanc::Logging::LogLevel_TRACE,                          \
+   ::Orthanc::Logging::LogCategory_GENERIC, __FILE__, __LINE__)
+#  define CLOG(level, category) ::Orthanc::Logging::InternalLogger      \
+  (::Orthanc::Logging::LogLevel_ ## level,                              \
+   ::Orthanc::Logging::LogCategory_ ## category, __FILE__, __LINE__)
 #endif
 
 
@@ -125,13 +186,25 @@ namespace Orthanc
     {
     private:
       LogLevel           level_;
+      LogCategory        category_;
       std::stringstream  messageStream_;
 
     public:
       InternalLogger(LogLevel level,
+                     LogCategory category,
                      const char* file  /* ignored */,
                      int line  /* ignored */) :
-        level_(level)
+        level_(level),
+        category_(category)
+      {
+      }
+
+      // For backward binary compatibility with Orthanc Framework <= 1.8.0
+      InternalLogger(LogLevel level,
+                     const char* file  /* ignored */,
+                     int line  /* ignored */) :
+        level_(level),
+        category_(LogCategory_GENERIC)
       {
       }
 
@@ -153,8 +226,6 @@ namespace Orthanc
 #if (ORTHANC_ENABLE_LOGGING == 1 &&             \
      ORTHANC_ENABLE_LOGGING_STDIO == 0)
 
-#include "Compatibility.h"  // For std::unique_ptr<>
-
 #include <boost/lexical_cast.hpp>
 #include <boost/noncopyable.hpp>
 #include <boost/thread/mutex.hpp>
@@ -172,7 +243,17 @@ namespace Orthanc
       std::unique_ptr<std::stringstream>  pluginStream_;
       std::ostream*                       stream_;
 
+      void Setup(LogCategory category,
+                 const char* file,
+                 int line);
+
     public:
+      InternalLogger(LogLevel level,
+                     LogCategory category,
+                     const char* file,
+                     int line);
+
+      // For backward binary compatibility with Orthanc Framework <= 1.8.0
       InternalLogger(LogLevel level,
                      const char* file,
                      int line);
@@ -186,7 +267,6 @@ namespace Orthanc
       }
     };
 
-
     /**
      * Set custom logging streams for the error, warning and info
      * logs. This function may not be called if a log file or folder
