@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -47,6 +48,10 @@ namespace Orthanc
     static unsigned int ReadUnsignedInteger(const Json::Value& value,
                                             const std::string& field);
 
+    static unsigned int ReadUnsignedInteger(const Json::Value& value,
+                                            const std::string& field,
+                                            unsigned int defaultValue);
+
     static bool ReadBoolean(const Json::Value& value,
                             const std::string& field);
 
@@ -66,12 +71,12 @@ namespace Orthanc
                               const Json::Value& value,
                               const std::string& field);
 
-    static void ReadMapOfStrings(std::map<std::string, std::string>& values,
-                                 const Json::Value& target,
+    static void ReadMapOfStrings(std::map<std::string, std::string>& target,
+                                 const Json::Value& value,
                                  const std::string& field);
 
-    static void ReadMapOfTags(std::map<DicomTag, std::string>& values,
-                              const Json::Value& target,
+    static void ReadMapOfTags(std::map<DicomTag, std::string>& target,
+                              const Json::Value& value,
                               const std::string& field);
 
     static void WriteArrayOfStrings(Json::Value& target,
@@ -97,5 +102,44 @@ namespace Orthanc
     static void WriteMapOfTags(Json::Value& target,
                                const std::map<DicomTag, std::string>& values,
                                const std::string& field);
+
+    static bool ParseInteger32(int32_t& result,
+                               const std::string& value);
+
+    static bool ParseInteger64(int64_t& result,
+                               const std::string& value);
+
+    static bool ParseUnsignedInteger32(uint32_t& result,
+                                       const std::string& value);
+
+    static bool ParseUnsignedInteger64(uint64_t& result,
+                                       const std::string& value);
+
+    static bool ParseFloat(float& result,
+                           const std::string& value);
+
+    static bool ParseDouble(double& result,
+                            const std::string& value);
+
+    static bool ParseFirstInteger32(int32_t& result,
+                                    const std::string& value);
+
+    static bool ParseFirstInteger64(int64_t& result,
+                                    const std::string& value);
+
+    static bool ParseFirstUnsignedInteger32(uint32_t& result,
+                                            const std::string& value);
+
+    static bool ParseFirstUnsignedInteger64(uint64_t& result,
+                                            const std::string& value);
+
+    static bool ParseFirstFloat(float& result,
+                                const std::string& value);
+
+    static bool ParseFirstDouble(double& result,
+                                 const std::string& value);
+
+    static bool ParseBoolean(bool& result,
+                             const std::string& value);
   };
 }
