@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -47,7 +48,23 @@ namespace Orthanc
     if (handle_ == NULL)
     {
       LOG(ERROR) << "LoadLibrary(" << path_ << ") failed: Error " << ::GetLastError();
-      throw OrthancException(ErrorCode_SharedLibrary);
+
+      if (::GetLastError() == ERROR_BAD_EXE_FORMAT &&
+          sizeof(void*) == 4)
+      {
+        throw OrthancException(ErrorCode_SharedLibrary,
+                               "You are most probably trying to load a 64bit plugin into a 32bit version of Orthanc");
+      }
+      else if (::GetLastError() == ERROR_BAD_EXE_FORMAT &&
+               sizeof(void*) == 8)
+      {
+        throw OrthancException(ErrorCode_SharedLibrary,
+                               "You are most probably trying to load a 32bit plugin into a 64bit version of Orthanc");
+      }
+      else
+      {
+        throw OrthancException(ErrorCode_SharedLibrary);
+      }
     }
 
 #elif defined(__linux__) || (defined(__APPLE__) && defined(__MACH__)) || defined(__FreeBSD_kernel__) || defined(__FreeBSD__) || defined(__OpenBSD__)
@@ -102,6 +119,12 @@ namespace Orthanc
   }
 
 
+  const std::string &SharedLibrary::GetPath() const
+  {
+    return path_;
+  }
+
+
   SharedLibrary::FunctionPointer SharedLibrary::GetFunctionInternal(const std::string& name)
   {
     if (!handle_)
