@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2021 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -46,6 +47,7 @@ namespace Orthanc
   static const char* KEY_URL = "Url";
   static const char* KEY_URL_2 = "URL";
   static const char* KEY_USERNAME = "Username";
+  static const char* KEY_TIMEOUT = "Timeout";
 
 
   static bool IsReservedKey(const std::string& key)
@@ -58,12 +60,14 @@ namespace Orthanc
             key == KEY_PKCS11 ||
             key == KEY_URL ||
             key == KEY_URL_2 ||
-            key == KEY_USERNAME);
+            key == KEY_USERNAME ||
+            key == KEY_TIMEOUT);
   }
 
 
   WebServiceParameters::WebServiceParameters() : 
-    pkcs11Enabled_(false)
+    pkcs11Enabled_(false),
+    timeout_(0)
   {
     SetUrl("http://127.0.0.1:8042/");
   }
@@ -214,6 +218,7 @@ namespace Orthanc
     assert(peer.isArray());
 
     pkcs11Enabled_ = false;
+    timeout_ = 0;
     ClearClientCertificate();
 
     if (peer.size() != 1 && 
@@ -366,6 +371,16 @@ namespace Orthanc
         }
       }
     }
+
+
+    if (peer.isMember(KEY_TIMEOUT))
+    {
+      timeout_ = SerializationToolbox::ReadUnsignedInteger(peer, KEY_TIMEOUT);      
+    }
+    else
+    {
+      timeout_ = 0;
+    }
   }
 
 
@@ -490,21 +505,19 @@ namespace Orthanc
     {
       return defaultValue;
     }
-    else if (found->second == "0" ||
-             found->second == "false")
-    {
-      return false;
-    }
-    else if (found->second == "1" ||
-             found->second == "true")
-    {
-      return true;
-    }
     else
     {
-      throw OrthancException(ErrorCode_BadFileFormat, "Bad value for a Boolean user property in the parameters "
-                             "of a Web service: Property \"" + key + "\" equals: " + found->second);
-    }    
+      bool value;
+      if (SerializationToolbox::ParseBoolean(value, found->second))
+      {
+        return value;
+      }
+      else
+      {
+        throw OrthancException(ErrorCode_BadFileFormat, "Bad value for a Boolean user property in the parameters "
+                               "of a Web service: Property \"" + key + "\" equals: " + found->second);
+      }
+    }
   }
 
 
@@ -515,7 +528,8 @@ namespace Orthanc
             !certificateKeyPassword_.empty() ||
             pkcs11Enabled_ ||
             !headers_.empty() ||
-            !userProperties_.empty());
+            !userProperties_.empty() ||
+            timeout_ != 0);
   }
 
 
@@ -557,6 +571,7 @@ namespace Orthanc
       }
 
       value[KEY_PKCS11] = pkcs11Enabled_;
+      value[KEY_TIMEOUT] = timeout_;
 
       value[KEY_HTTP_HEADERS] = Json::objectValue;
       for (Dictionary::const_iterator it = headers_.begin();
@@ -631,6 +646,7 @@ namespace Orthanc
     }
 
     target[KEY_PKCS11] = pkcs11Enabled_;
+    target[KEY_TIMEOUT] = timeout_;
 
     Json::Value headers = Json::arrayValue;
       
@@ -649,4 +665,20 @@ namespace Orthanc
       target[it->first] = it->second;
     }
   }
+
+
+  void WebServiceParameters::SetTimeout(uint32_t seconds)
+  {
+    timeout_ = seconds;
+  }
+
+  uint32_t WebServiceParameters::GetTimeout() const
+  {
+    return timeout_;
+  }
+
+  bool WebServiceParameters::HasTimeout() const
+  {
+    return (timeout_ != 0);
+  }
 }
