@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2021 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -125,6 +126,7 @@ namespace
 
     virtual bool GetOutput(std::string& output,
                            MimeType& mime,
+                           std::string& filename,
                            const std::string& key) ORTHANC_OVERRIDE
     {
       return false;
@@ -1090,6 +1092,32 @@ TEST(JobsSerialization, DicomModification)
 }
 
 
+TEST(JobsSerialization, DicomModification2)
+{   
+  Json::Value s;
+
+  {
+    DicomModification modification;
+    modification.SetupAnonymization(DicomVersion_2017c);
+    modification.Remove(DicomPath(DICOM_TAG_REFERENCED_IMAGE_SEQUENCE, 1, DICOM_TAG_SOP_INSTANCE_UID));
+    modification.Replace(DicomPath(DICOM_TAG_REFERENCED_IMAGE_SEQUENCE, 1, DICOM_TAG_SOP_CLASS_UID), "Hello", true);
+    modification.Keep(DicomPath(DICOM_TAG_REFERENCED_IMAGE_SEQUENCE, 1, DICOM_TAG_PATIENT_NAME));
+
+    s = 42;
+    modification.Serialize(s);
+  }
+
+  {
+    DicomModification modification(s);
+
+    // Check idempotent serialization
+    Json::Value ss;
+    modification.Serialize(ss);
+    ASSERT_EQ(s.toStyledString(), ss.toStyledString());
+  }
+}
+
+
 TEST(JobsSerialization, Registry)
 {   
   Json::Value s;
@@ -1271,6 +1299,8 @@ TEST(JobsSerialization, RemoteModalityParameters)
     modality.Serialize(s, false);
     ASSERT_EQ(Json::arrayValue, s.type());
     ASSERT_FALSE(modality.IsDicomTlsEnabled());
+    ASSERT_FALSE(modality.HasTimeout());
+    ASSERT_EQ(0u, modality.GetTimeout());
   }
 
   {
@@ -1282,6 +1312,7 @@ TEST(JobsSerialization, RemoteModalityParameters)
     ASSERT_EQ(ModalityManufacturer_Generic, modality.GetManufacturer());
     ASSERT_TRUE(modality.IsRequestAllowed(DicomRequestType_Echo));
     ASSERT_TRUE(modality.IsRequestAllowed(DicomRequestType_Find));
+    ASSERT_TRUE(modality.IsRequestAllowed(DicomRequestType_FindWorklist));
     ASSERT_TRUE(modality.IsRequestAllowed(DicomRequestType_Get));
     ASSERT_TRUE(modality.IsRequestAllowed(DicomRequestType_Store));
     ASSERT_TRUE(modality.IsRequestAllowed(DicomRequestType_Move));
@@ -1291,6 +1322,8 @@ TEST(JobsSerialization, RemoteModalityParameters)
     ASSERT_FALSE(modality.IsDicomTlsEnabled());
     ASSERT_FALSE(modality.HasLocalAet());
     ASSERT_THROW(modality.GetLocalAet(), OrthancException);
+    ASSERT_FALSE(modality.HasTimeout());
+    ASSERT_EQ(0u, modality.GetTimeout());
   }
 
   s = Json::nullValue;
@@ -1307,6 +1340,8 @@ TEST(JobsSerialization, RemoteModalityParameters)
     modality.Serialize(s, true);
     ASSERT_EQ(Json::objectValue, s.type());
     ASSERT_FALSE(modality.HasLocalAet());
+    ASSERT_FALSE(modality.HasTimeout());
+    ASSERT_EQ(0u, modality.GetTimeout());
   }
 
   {
@@ -1317,6 +1352,7 @@ TEST(JobsSerialization, RemoteModalityParameters)
     ASSERT_EQ(ModalityManufacturer_GenericNoWildcardInDates, modality.GetManufacturer());
     ASSERT_TRUE(modality.IsRequestAllowed(DicomRequestType_Echo));
     ASSERT_TRUE(modality.IsRequestAllowed(DicomRequestType_Find));
+    ASSERT_TRUE(modality.IsRequestAllowed(DicomRequestType_FindWorklist));
     ASSERT_TRUE(modality.IsRequestAllowed(DicomRequestType_Get));
     ASSERT_TRUE(modality.IsRequestAllowed(DicomRequestType_Store));
     ASSERT_TRUE(modality.IsRequestAllowed(DicomRequestType_Move));
@@ -1325,6 +1361,8 @@ TEST(JobsSerialization, RemoteModalityParameters)
     ASSERT_TRUE(modality.IsTranscodingAllowed());
     ASSERT_FALSE(modality.IsDicomTlsEnabled());
     ASSERT_FALSE(modality.HasLocalAet());
+    ASSERT_FALSE(modality.HasTimeout());
+    ASSERT_EQ(0u, modality.GetTimeout());
   }
 
   s["Port"] = "46";
@@ -1341,13 +1379,14 @@ TEST(JobsSerialization, RemoteModalityParameters)
   std::set<DicomRequestType> operations;
   operations.insert(DicomRequestType_Echo);
   operations.insert(DicomRequestType_Find);
+  operations.insert(DicomRequestType_FindWorklist);
   operations.insert(DicomRequestType_Get);
   operations.insert(DicomRequestType_Move);
   operations.insert(DicomRequestType_Store);
   operations.insert(DicomRequestType_NAction);
   operations.insert(DicomRequestType_NEventReport);
 
-  ASSERT_EQ(7u, operations.size());
+  ASSERT_EQ(8u, operations.size());
 
   for (std::set<DicomRequestType>::const_iterator 
          it = operations.begin(); it != operations.end(); ++it)
@@ -1382,16 +1421,21 @@ TEST(JobsSerialization, RemoteModalityParameters)
   {
     RemoteModalityParameters modality;
     modality.SetLocalAet("hello");
+    modality.SetTimeout(42);
     ASSERT_TRUE(modality.IsAdvancedFormatNeeded());
     modality.Serialize(s, true);
     ASSERT_EQ(Json::objectValue, s.type());
     ASSERT_TRUE(modality.HasLocalAet());
+    ASSERT_TRUE(modality.HasTimeout());
+    ASSERT_EQ(42u, modality.GetTimeout());
   }
 
   {
     RemoteModalityParameters modality(s);
     ASSERT_TRUE(modality.HasLocalAet());
     ASSERT_EQ("hello", modality.GetLocalAet());
+    ASSERT_TRUE(modality.HasTimeout());
+    ASSERT_EQ(42u, modality.GetTimeout());
   }
 
   {
@@ -1412,6 +1456,8 @@ TEST(JobsSerialization, RemoteModalityParameters)
     ASSERT_FALSE(modality.IsDicomTlsEnabled());
     ASSERT_FALSE(modality.HasLocalAet());
     ASSERT_THROW(modality.GetLocalAet(), OrthancException);
+    ASSERT_FALSE(modality.HasTimeout());
+    ASSERT_EQ(0u, modality.GetTimeout());
   }
 
   {
@@ -1424,6 +1470,7 @@ TEST(JobsSerialization, RemoteModalityParameters)
     t["AllowTranscoding"] = false;
     t["UseDicomTls"] = true;
     t["LocalAet"] = "world";
+    t["Timeout"] = 20;
     
     RemoteModalityParameters modality(t);
     ASSERT_TRUE(modality.IsAdvancedFormatNeeded());
@@ -1436,6 +1483,8 @@ TEST(JobsSerialization, RemoteModalityParameters)
     ASSERT_TRUE(modality.IsDicomTlsEnabled());
     ASSERT_TRUE(modality.HasLocalAet());
     ASSERT_EQ("world", modality.GetLocalAet());
+    ASSERT_TRUE(modality.HasTimeout());
+    ASSERT_EQ(20u, modality.GetTimeout());
   }
 
   {
@@ -1475,7 +1524,7 @@ TEST(JobsSerialization, DicomAssociationParameters)
     ASSERT_TRUE(v.isMember("Remote"));
     ASSERT_TRUE(v.isMember("MaximumPduLength"));
 
-    ASSERT_EQ(4u, v.getMemberNames().size());
+    ASSERT_EQ(5u, v.getMemberNames().size());
   
     DicomAssociationParameters b;
     b.UnserializeJob(v);
@@ -1487,6 +1536,9 @@ TEST(JobsSerialization, DicomAssociationParameters)
     ASSERT_FALSE(b.GetRemoteModality().IsDicomTlsEnabled());
     ASSERT_FALSE(b.GetRemoteModality().HasLocalAet());
     ASSERT_THROW(b.GetRemoteModality().GetLocalAet(), OrthancException);
+    ASSERT_FALSE(b.GetRemoteModality().HasTimeout());
+    ASSERT_EQ(0u, b.GetRemoteModality().GetTimeout());
+    ASSERT_TRUE(b.IsRemoteCertificateRequired());
   }
 
   {
@@ -1495,10 +1547,12 @@ TEST(JobsSerialization, DicomAssociationParameters)
     p.SetPortNumber(4242);
     p.SetHost("hello.world.com");
     p.SetDicomTlsEnabled(true);
+    p.SetTimeout(42);
     
     DicomAssociationParameters a("HELLO", p);
     a.SetOwnCertificatePath("key", "crt");
     a.SetTrustedCertificatesPath("trusted");
+    a.SetRemoteCertificateRequired(false);
 
     ASSERT_THROW(a.SetMaximumPduLength(4095), OrthancException);
     ASSERT_THROW(a.SetMaximumPduLength(131073), OrthancException);
@@ -1508,7 +1562,7 @@ TEST(JobsSerialization, DicomAssociationParameters)
     Json::Value v = Json::objectValue;
     a.SerializeJob(v);
 
-    ASSERT_EQ(7u, v.getMemberNames().size());
+    ASSERT_EQ(8u, v.getMemberNames().size());
   
     DicomAssociationParameters b = DicomAssociationParameters::UnserializeJob(v);
 
@@ -1521,5 +1575,91 @@ TEST(JobsSerialization, DicomAssociationParameters)
     ASSERT_EQ("crt", b.GetOwnCertificatePath());
     ASSERT_EQ("trusted", b.GetTrustedCertificatesPath());
     ASSERT_EQ(131072u, b.GetMaximumPduLength());
+    ASSERT_TRUE(b.GetRemoteModality().HasTimeout());
+    ASSERT_EQ(42u, b.GetRemoteModality().GetTimeout());
+    ASSERT_FALSE(b.IsRemoteCertificateRequired());
   }  
 }
+
+
+TEST(SerializationToolbox, Numbers)
+{
+  {
+    int32_t i;
+    ASSERT_FALSE(SerializationToolbox::ParseInteger32(i, ""));
+    ASSERT_FALSE(SerializationToolbox::ParseInteger32(i, "ee"));
+    ASSERT_TRUE(SerializationToolbox::ParseInteger32(i, "42"));  ASSERT_EQ(42, i);
+    ASSERT_TRUE(SerializationToolbox::ParseInteger32(i, "-42"));  ASSERT_EQ(-42, i);
+    ASSERT_TRUE(SerializationToolbox::ParseInteger32(i, "-2147483648")); ASSERT_EQ(-2147483648l, i);
+    ASSERT_TRUE(SerializationToolbox::ParseInteger32(i, "2147483647")); ASSERT_EQ(2147483647l, i);
+    ASSERT_FALSE(SerializationToolbox::ParseInteger32(i, "-2147483649"));
+    ASSERT_FALSE(SerializationToolbox::ParseInteger32(i, "2147483648"));
+    ASSERT_FALSE(SerializationToolbox::ParseInteger32(i, "-2\\-3\\-4"));
+    ASSERT_TRUE(SerializationToolbox::ParseFirstInteger32(i, "-2\\-3\\-4"));  ASSERT_EQ(-2, i);
+  }
+
+  {
+    uint32_t i;
+    ASSERT_FALSE(SerializationToolbox::ParseUnsignedInteger32(i, ""));
+    ASSERT_FALSE(SerializationToolbox::ParseUnsignedInteger32(i, "ee"));
+    ASSERT_TRUE(SerializationToolbox::ParseUnsignedInteger32(i, "42"));  ASSERT_EQ(42u, i);
+    ASSERT_FALSE(SerializationToolbox::ParseUnsignedInteger32(i, "-42"));
+    ASSERT_TRUE(SerializationToolbox::ParseUnsignedInteger32(i, "4294967295")); ASSERT_EQ(4294967295u, i);
+    ASSERT_FALSE(SerializationToolbox::ParseUnsignedInteger32(i, "4294967296"));
+    ASSERT_FALSE(SerializationToolbox::ParseUnsignedInteger32(i, "2\\3\\4"));
+    ASSERT_TRUE(SerializationToolbox::ParseFirstUnsignedInteger32(i, "2\\3\\4"));  ASSERT_EQ(2u, i);
+  }
+
+  {
+    int64_t i;
+    ASSERT_FALSE(SerializationToolbox::ParseInteger64(i, ""));
+    ASSERT_FALSE(SerializationToolbox::ParseInteger64(i, "ee"));
+    ASSERT_TRUE(SerializationToolbox::ParseInteger64(i, "42"));  ASSERT_EQ(42, i);
+    ASSERT_TRUE(SerializationToolbox::ParseInteger64(i, "-42"));  ASSERT_EQ(-42, i);
+    ASSERT_TRUE(SerializationToolbox::ParseInteger64(i, "-2147483649")); ASSERT_EQ(-2147483649ll, i);
+    ASSERT_TRUE(SerializationToolbox::ParseInteger64(i, "2147483648")); ASSERT_EQ(2147483648ll, i);
+    ASSERT_FALSE(SerializationToolbox::ParseInteger64(i, "-2\\-3\\-4"));
+    ASSERT_TRUE(SerializationToolbox::ParseFirstInteger64(i, "-2\\-3\\-4"));  ASSERT_EQ(-2, i);
+  }
+
+  {
+    uint64_t i;
+    ASSERT_FALSE(SerializationToolbox::ParseUnsignedInteger64(i, ""));
+    ASSERT_FALSE(SerializationToolbox::ParseUnsignedInteger64(i, "ee"));
+    ASSERT_TRUE(SerializationToolbox::ParseUnsignedInteger64(i, "42"));  ASSERT_EQ(42u, i);
+    ASSERT_FALSE(SerializationToolbox::ParseUnsignedInteger64(i, "-42"));
+    ASSERT_TRUE(SerializationToolbox::ParseUnsignedInteger64(i, "4294967296")); ASSERT_EQ(4294967296lu, i);
+    ASSERT_FALSE(SerializationToolbox::ParseUnsignedInteger64(i, "2\\3\\4"));
+    ASSERT_TRUE(SerializationToolbox::ParseFirstUnsignedInteger64(i, "2\\3\\4"));  ASSERT_EQ(2u, i);
+  }
+
+  {
+    float i;
+    ASSERT_FALSE(SerializationToolbox::ParseFloat(i, ""));
+    ASSERT_FALSE(SerializationToolbox::ParseFloat(i, "ee"));
+    ASSERT_TRUE(SerializationToolbox::ParseFloat(i, "42"));  ASSERT_FLOAT_EQ(42.0f, i);
+    ASSERT_TRUE(SerializationToolbox::ParseFloat(i, "-42"));  ASSERT_FLOAT_EQ(-42.0f, i);
+    ASSERT_FALSE(SerializationToolbox::ParseFloat(i, "2\\3\\4"));
+    ASSERT_TRUE(SerializationToolbox::ParseFirstFloat(i, "1.367\\2.367\\3.367"));  ASSERT_FLOAT_EQ(1.367f, i);
+
+    ASSERT_TRUE(SerializationToolbox::ParseFloat(i, "1.2"));  ASSERT_FLOAT_EQ(1.2f, i);
+    ASSERT_TRUE(SerializationToolbox::ParseFloat(i, "-1.2e+2"));  ASSERT_FLOAT_EQ(-120.0f, i);
+    ASSERT_TRUE(SerializationToolbox::ParseFloat(i, "-1e-2"));  ASSERT_FLOAT_EQ(-0.01f, i);
+    ASSERT_TRUE(SerializationToolbox::ParseFloat(i, "1.3671875"));  ASSERT_FLOAT_EQ(1.3671875f, i);
+  }
+
+  {
+    double i;
+    ASSERT_FALSE(SerializationToolbox::ParseDouble(i, ""));
+    ASSERT_FALSE(SerializationToolbox::ParseDouble(i, "ee"));
+    ASSERT_TRUE(SerializationToolbox::ParseDouble(i, "42"));  ASSERT_DOUBLE_EQ(42.0, i);
+    ASSERT_TRUE(SerializationToolbox::ParseDouble(i, "-42"));  ASSERT_DOUBLE_EQ(-42.0, i);
+    ASSERT_FALSE(SerializationToolbox::ParseDouble(i, "2\\3\\4"));
+    ASSERT_TRUE(SerializationToolbox::ParseFirstDouble(i, "1.367\\2.367\\3.367"));  ASSERT_DOUBLE_EQ(1.367, i);
+
+    ASSERT_TRUE(SerializationToolbox::ParseDouble(i, "1.2"));  ASSERT_DOUBLE_EQ(1.2, i);
+    ASSERT_TRUE(SerializationToolbox::ParseDouble(i, "-1.2e+2"));  ASSERT_DOUBLE_EQ(-120.0, i);
+    ASSERT_TRUE(SerializationToolbox::ParseDouble(i, "-1e-2"));  ASSERT_DOUBLE_EQ(-0.01, i);
+    ASSERT_TRUE(SerializationToolbox::ParseDouble(i, "1.3671875"));  ASSERT_DOUBLE_EQ(1.3671875, i);
+  }
+}
