@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -252,7 +253,7 @@ namespace
     std::string value_;
 
   public:
-    S(const std::string& value) : value_(value)
+    explicit S(const std::string& value) : value_(value)
     {
     }
 
@@ -300,7 +301,7 @@ TEST(LRU, SharedArchive)
 
   size_t count = 0;
   for (std::list<std::string>::const_iterator
-         it = i.begin(); it != i.end(); it++)
+         it = i.begin(); it != i.end(); ++it)
   {
     if (*it == first ||
         *it == second)
