@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2021 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -31,6 +32,7 @@
 #include "../Sources/Compression/ZlibCompressor.h"
 #include "../Sources/HttpServer/HttpContentNegociation.h"
 #include "../Sources/HttpServer/MultipartStreamReader.h"
+#include "../Sources/HttpServer/StringMatcher.h"
 #include "../Sources/Logging.h"
 #include "../Sources/OrthancException.h"
 #include "../Sources/RestApi/RestApiHierarchy.h"
@@ -568,11 +570,12 @@ TEST(WebServiceParameters, Serialization)
     p.Serialize(v2, false, true);
 
     ASSERT_EQ(Json::objectValue, v2.type());
-    ASSERT_EQ(3u, v2.size());
+    ASSERT_EQ(4u, v2.size());
     ASSERT_EQ("http://localhost:8042/", v2["Url"].asString());
     ASSERT_TRUE(v2["Pkcs11"].asBool());
     ASSERT_EQ(Json::objectValue, v2["HttpHeaders"].type());
     ASSERT_EQ(0u, v2["HttpHeaders"].size());
+    ASSERT_EQ(0, v2["Timeout"].asInt());
 
     WebServiceParameters p2(v2);  // Test decoding
     ASSERT_EQ("http://localhost:8042/", p2.GetUrl());
@@ -591,7 +594,7 @@ TEST(WebServiceParameters, Serialization)
     p.Serialize(v2, false, true);
 
     ASSERT_EQ(Json::objectValue, v2.type());
-    ASSERT_EQ(6u, v2.size());
+    ASSERT_EQ(7u, v2.size());
     ASSERT_EQ("http://localhost:8042/", v2["Url"].asString());
     ASSERT_EQ("a", v2["CertificateFile"].asString());
     ASSERT_EQ("b", v2["CertificateKeyFile"].asString());
@@ -599,6 +602,7 @@ TEST(WebServiceParameters, Serialization)
     ASSERT_FALSE(v2["Pkcs11"].asBool());
     ASSERT_EQ(Json::objectValue, v2["HttpHeaders"].type());
     ASSERT_EQ(0u, v2["HttpHeaders"].size());
+    ASSERT_EQ(0, v2["Timeout"].asInt());
 
     WebServiceParameters p2(v2);  // Test decoding
     ASSERT_EQ("http://localhost:8042/", p2.GetUrl());
@@ -612,6 +616,7 @@ TEST(WebServiceParameters, Serialization)
     ASSERT_FALSE(p.IsAdvancedFormatNeeded());
     p.AddHttpHeader("a", "b");
     p.AddHttpHeader("c", "d");
+    p.SetTimeout(42);
     ASSERT_TRUE(p.IsAdvancedFormatNeeded());
 
     Json::Value v2;
@@ -619,13 +624,14 @@ TEST(WebServiceParameters, Serialization)
     WebServiceParameters p2(v2);
 
     ASSERT_EQ(Json::objectValue, v2.type());
-    ASSERT_EQ(3u, v2.size());
+    ASSERT_EQ(4u, v2.size());
     ASSERT_EQ("http://localhost:8042/", v2["Url"].asString());
     ASSERT_FALSE(v2["Pkcs11"].asBool());
     ASSERT_EQ(Json::objectValue, v2["HttpHeaders"].type());
     ASSERT_EQ(2u, v2["HttpHeaders"].size());
     ASSERT_EQ("b", v2["HttpHeaders"]["a"].asString());
     ASSERT_EQ("d", v2["HttpHeaders"]["c"].asString());
+    ASSERT_EQ(42, v2["Timeout"].asInt());
 
     std::set<std::string> a;
     p2.ListHttpHeaders(a);
@@ -718,7 +724,6 @@ TEST(StringMatcher, Basic)
   {
     std::string s(10u, '\0');  // String with null values
     ASSERT_EQ(10u, s.size());
-    ASSERT_EQ(10u, s.size());
     ASSERT_FALSE(matcher.Apply(s));
 
     s[9] = '-';
@@ -737,6 +742,89 @@ TEST(StringMatcher, Basic)
 }
 
 
+TEST(CStringMatcher, Basic)
+{
+  CStringMatcher matcher("---");
+
+  ASSERT_THROW(matcher.GetMatchBegin(), OrthancException);
+
+  {
+    ASSERT_FALSE(matcher.Apply(NULL, 0));
+    
+    const std::string s = "";
+    ASSERT_FALSE(matcher.Apply(s));
+  }
+
+  {
+    const char* s = "abc---def";
+    ASSERT_TRUE(matcher.Apply(s, s + 9));
+    
+    ASSERT_EQ('a', matcher.GetMatchBegin()[-3]);
+    ASSERT_EQ('b', matcher.GetMatchBegin()[-2]);
+    ASSERT_EQ('c', matcher.GetMatchBegin()[-1]);
+    ASSERT_EQ('-', matcher.GetMatchBegin()[0]);
+    ASSERT_EQ('-', matcher.GetMatchBegin()[1]);
+    ASSERT_EQ('-', matcher.GetMatchBegin()[2]);
+    ASSERT_EQ('d', matcher.GetMatchBegin()[3]);
+    ASSERT_EQ('e', matcher.GetMatchBegin()[4]);
+    ASSERT_EQ('f', matcher.GetMatchBegin()[5]);
+    ASSERT_EQ('\0', matcher.GetMatchBegin()[6]);
+
+    ASSERT_EQ('a', matcher.GetMatchEnd()[-6]);
+    ASSERT_EQ('b', matcher.GetMatchEnd()[-5]);
+    ASSERT_EQ('c', matcher.GetMatchEnd()[-4]);
+    ASSERT_EQ('-', matcher.GetMatchEnd()[-3]);
+    ASSERT_EQ('-', matcher.GetMatchEnd()[-2]);
+    ASSERT_EQ('-', matcher.GetMatchEnd()[-1]);
+    ASSERT_EQ('d', matcher.GetMatchEnd()[0]);
+    ASSERT_EQ('e', matcher.GetMatchEnd()[1]);
+    ASSERT_EQ('f', matcher.GetMatchEnd()[2]);
+    ASSERT_EQ('\0', matcher.GetMatchEnd()[3]);
+  }
+
+  {
+    const std::string s = "abc----def";
+    ASSERT_TRUE(matcher.Apply(s));
+    ASSERT_EQ(3, std::distance(s.c_str(), matcher.GetMatchBegin()));
+    ASSERT_EQ("---", std::string(matcher.GetMatchBegin(), matcher.GetMatchEnd()));
+  }
+
+  {
+    const std::string s = "abc---";
+    ASSERT_TRUE(matcher.Apply(s));
+    ASSERT_EQ(3, std::distance(s.c_str(), matcher.GetMatchBegin()));
+    ASSERT_EQ(s.c_str() + s.size(), matcher.GetMatchEnd());
+    ASSERT_EQ("---", std::string(matcher.GetMatchBegin(), matcher.GetMatchEnd()));
+    ASSERT_EQ("", std::string(matcher.GetMatchEnd(), s.c_str() + s.size()));
+  }
+
+  {
+    const std::string s = "abc--def";
+    ASSERT_FALSE(matcher.Apply(s));
+    ASSERT_THROW(matcher.GetMatchBegin(), OrthancException);
+    ASSERT_THROW(matcher.GetMatchEnd(), OrthancException);
+  }
+
+  {
+    std::string s(10u, '\0');  // String with null values
+    ASSERT_EQ(10u, s.size());
+    ASSERT_FALSE(matcher.Apply(s));
+
+    s[9] = '-';
+    ASSERT_FALSE(matcher.Apply(s));
+
+    s[8] = '-';
+    ASSERT_FALSE(matcher.Apply(s));
+
+    s[7] = '-';
+    ASSERT_TRUE(matcher.Apply(s));
+    ASSERT_EQ(s.c_str() + 7, matcher.GetMatchBegin());
+    ASSERT_EQ(s.c_str() + 10, matcher.GetMatchEnd());
+    ASSERT_EQ(s.c_str() + s.size() - 3, matcher.GetMatchBegin());
+    ASSERT_EQ(s.c_str() + s.size(), matcher.GetMatchEnd());
+  }
+}
+
 
 class MultipartTester : public MultipartStreamReader::IHandler
 {
