@@ -12,22 +12,25 @@ cat <<EOF > /tmp/cppcheck-suppressions.txt
 constParameter:../../OrthancFramework/Sources/DicomParsing/FromDcmtkBridge.cpp
 knownArgument:../../OrthancFramework/UnitTestsSources/ImageTests.cpp
 knownConditionTrueFalse:../../OrthancServer/Plugins/Engine/OrthancPlugins.cpp
-nullPointer:../../OrthancFramework/UnitTestsSources/RestApiTests.cpp:319
+nullPointer:../../OrthancFramework/UnitTestsSources/RestApiTests.cpp:321
 nullPointerRedundantCheck:../../OrthancFramework/UnitTestsSources/DicomMapTests.cpp
-stlFindInsert:../../OrthancFramework/Sources/DicomFormat/DicomMap.cpp:1194
-stlFindInsert:../../OrthancFramework/Sources/RestApi/RestApiCallDocumentation.cpp:164
-stlFindInsert:../../OrthancFramework/Sources/RestApi/RestApiCallDocumentation.cpp:72
-stlFindInsert:../../OrthancServer/Sources/Database/StatelessDatabaseOperations.cpp:384
-stlFindInsert:../../OrthancServer/Sources/OrthancWebDav.cpp:386
-syntaxError:../../OrthancFramework/Sources/SQLite/FunctionContext.h:50
-syntaxError:../../OrthancFramework/UnitTestsSources/ZipTests.cpp:130
-syntaxError:../../OrthancServer/UnitTestsSources/UnitTestsMain.cpp:321
-uninitMemberVar:../../OrthancServer/Sources/ServerJobs/StorageCommitmentScpJob.cpp:427
+stlFindInsert:../../OrthancFramework/Sources/DicomFormat/DicomMap.cpp:1195
+stlFindInsert:../../OrthancFramework/Sources/RestApi/RestApiCallDocumentation.cpp:165
+stlFindInsert:../../OrthancFramework/Sources/RestApi/RestApiCallDocumentation.cpp:73
+stlFindInsert:../../OrthancServer/Sources/Database/StatelessDatabaseOperations.cpp:373
+stlFindInsert:../../OrthancServer/Sources/OrthancWebDav.cpp:375
+stlFindInsert:../../OrthancServer/Sources/ServerJobs/MergeStudyJob.cpp:40
+stlFindInsert:../../OrthancServer/Sources/ServerJobs/SplitStudyJob.cpp:190
+syntaxError:../../OrthancFramework/Sources/SQLite/FunctionContext.h:52
+syntaxError:../../OrthancFramework/UnitTestsSources/ZipTests.cpp:132
+syntaxError:../../OrthancServer/UnitTestsSources/UnitTestsMain.cpp:310
+uninitMemberVar:../../OrthancServer/Sources/ServerJobs/StorageCommitmentScpJob.cpp:416
 unreadVariable:../../OrthancFramework/Sources/FileStorage/StorageAccessor.cpp
-unreadVariable:../../OrthancServer/Sources/OrthancRestApi/OrthancRestModalities.cpp:1121
+unreadVariable:../../OrthancServer/Sources/OrthancRestApi/OrthancRestModalities.cpp:1115
 unusedFunction
-useInitializationList:../../OrthancFramework/Sources/Images/PngReader.cpp:89
-useInitializationList:../../OrthancServer/Sources/ServerJobs/DicomModalityStoreJob.cpp:285
+useInitializationList:../../OrthancFramework/Sources/Images/PngReader.cpp:90
+useInitializationList:../../OrthancFramework/Sources/Images/PngWriter.cpp:98
+useInitializationList:../../OrthancServer/Sources/ServerJobs/DicomModalityStoreJob.cpp:274
 EOF
 
 ${CPPCHECK} --enable=all --quiet --std=c++11 \
