@@ -2,24 +2,13 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation, either version 3 of the
  * License, or (at your option) any later version.
- *
- * In addition, as a special exception, the copyright holders of this
- * program give permission to link the code of its release with the
- * OpenSSL project's "OpenSSL" library (or with modified versions of it
- * that use the same license as the "OpenSSL" library), and distribute
- * the linked executables. You must obey the GNU General Public License
- * in all respects for all of the code used other than "OpenSSL". If you
- * modify file(s) with this exception, you may extend this exception to
- * your version of the file(s), but you are not obligated to do so. If
- * you do not wish to do so, delete this exception statement from your
- * version. If you delete this exception statement from all source files
- * in the program, then also delete it here.
  * 
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -53,25 +42,25 @@
 
 namespace Orthanc
 {
-  namespace
+  static void ProgressCallback(void *callbackData,
+                               T_DIMSE_StoreProgress *progress,
+                               T_DIMSE_C_StoreRQ *req)
   {
-    // Anonymous namespace to avoid clashes between compilation modules
-    
-    static void GetSubOpProgressCallback(
-      void * /* callbackData == pointer to the "OrthancGetRequestHandler" object */,
-      T_DIMSE_StoreProgress *progress,
-      T_DIMSE_C_StoreRQ * /*req*/)
+    if (req != NULL &&
+        progress->state == DIMSE_StoreBegin)
     {
-      // SBL - no logging to be done here.
+      OFString str;
+      CLOG(TRACE, DICOM) << "Sending Store Request following a C-GET:" << std::endl
+                         << DIMSE_dumpMessage(str, *req, DIMSE_OUTGOING);
     }
   }
 
-  OrthancGetRequestHandler::Status
-  OrthancGetRequestHandler::DoNext(T_ASC_Association* assoc)
+
+  bool OrthancGetRequestHandler::DoNext(T_ASC_Association* assoc)
   {
     if (position_ >= instances_.size())
     {
-      return Status_Failure;
+      throw OrthancException(ErrorCode_ParameterOutOfRange);
     }
     
     const std::string& id = instances_[position_++];
@@ -79,9 +68,9 @@ namespace Orthanc
     std::string dicom;
     context_.ReadDicom(dicom, id);
     
-    if (dicom.size() <= 0)
+    if (dicom.empty())
     {
-      return Status_Failure;
+      throw OrthancException(ErrorCode_BadFileFormat);
     }
 
     std::unique_ptr<DcmFileFormat> parsed(
@@ -107,19 +96,7 @@ namespace Orthanc
     std::string sopClassUid(a.c_str());
     std::string sopInstanceUid(b.c_str());
     
-    OFCondition cond = PerformGetSubOp(assoc, sopClassUid, sopInstanceUid, parsed.release());
-    
-    if (getCancelled_)
-    {
-      LOG(INFO) << "C-GET SCP: Received C-Cancel RQ";
-    }
-    
-    if (cond.bad() || getCancelled_)
-    {
-      return Status_Failure;
-    }
-    
-    return Status_Success;
+    return PerformGetSubOp(assoc, sopClassUid, sopInstanceUid, parsed.release());
   }
 
   
@@ -160,20 +137,22 @@ namespace Orthanc
       while (pc)
       {
         DicomTransferSyntax transferSyntax;
-        if (pc->result == ASC_P_ACCEPTANCE &&
-            LookupTransferSyntax(transferSyntax, pc->acceptedTransferSyntax))
+        if (pc->result == ASC_P_ACCEPTANCE)
         {
-          VLOG(0) << "C-GET SCP accepted: SOP class " << sopClassUid
-                  << " with transfer syntax " << GetTransferSyntaxUid(transferSyntax);
-          if (std::string(pc->abstractSyntax) == sopClassUid)
+          if (LookupTransferSyntax(transferSyntax, pc->acceptedTransferSyntax))
           {
-            accepted[transferSyntax] = pc->presentationContextID;
+            /*CLOG(TRACE, DICOM) << "C-GET SCP accepted: SOP class " << pc->abstractSyntax
+              << " with transfer syntax " << GetTransferSyntaxUid(transferSyntax);*/
+            if (std::string(pc->abstractSyntax) == sopClassUid)
+            {
+              accepted[transferSyntax] = pc->presentationContextID;
+            }
+          }
+          else
+          {
+            CLOG(WARNING, DICOM) << "C-GET: Unknown transfer syntax received: "
+                                 << pc->acceptedTransferSyntax;
           }
-        }
-        else
-        {
-          LOG(WARNING) << "C-GET: Unknown transfer syntax received: "
-                       << pc->acceptedTransferSyntax;
         }
             
         pc = (DUL_PRESENTATIONCONTEXT*) LST_Next(l);
@@ -233,10 +212,10 @@ namespace Orthanc
   }                                                           
 
 
-  OFCondition OrthancGetRequestHandler::PerformGetSubOp(T_ASC_Association* assoc,
-                                                        const std::string& sopClassUid,
-                                                        const std::string& sopInstanceUid,
-                                                        DcmFileFormat* dicomRaw)
+  bool OrthancGetRequestHandler::PerformGetSubOp(T_ASC_Association* assoc,
+                                                 const std::string& sopClassUid,
+                                                 const std::string& sopInstanceUid,
+                                                 DcmFileFormat* dicomRaw)
   {
     assert(dicomRaw != NULL);
     std::unique_ptr<DcmFileFormat> dicom(dicomRaw);
@@ -244,33 +223,32 @@ namespace Orthanc
     DicomTransferSyntax sourceSyntax;
     if (!FromDcmtkBridge::LookupOrthancTransferSyntax(sourceSyntax, *dicom))
     {
-      nFailed_++;
+      failedCount_++;
       AddFailedUIDInstance(sopInstanceUid);
-      LOG(ERROR) << "C-GET SCP: Unknown transfer syntax: ("
-                 << dcmSOPClassUIDToModality(sopClassUid.c_str(), "OT") << ") " << sopClassUid;
-      return DIMSE_NOVALIDPRESENTATIONCONTEXTID;
+      throw OrthancException(ErrorCode_NetworkProtocol, 
+                             "C-GET SCP: Unknown transfer syntax: (" +
+                             std::string(dcmSOPClassUIDToModality(sopClassUid.c_str(), "OT")) +
+                             ") " + sopClassUid);
     }
 
-    bool allowTranscoding = (context_.IsTranscodeDicomProtocol() &&
-                             remote_.IsTranscodingAllowed());
-    
     T_ASC_PresentationContextID presId = 0;  // Unnecessary initialization, makes code clearer
     DicomTransferSyntax selectedSyntax;
     if (!SelectPresentationContext(presId, selectedSyntax, assoc, sopClassUid,
-                                   sourceSyntax, allowTranscoding) ||
+                                   sourceSyntax, allowTranscoding_) ||
         presId == 0)
     {
-      nFailed_++;
+      failedCount_++;
       AddFailedUIDInstance(sopInstanceUid);
-      LOG(ERROR) << "C-GET SCP: storeSCU: No presentation context for: ("
-                 << dcmSOPClassUIDToModality(sopClassUid.c_str(), "OT") << ") " << sopClassUid;
-      return DIMSE_NOVALIDPRESENTATIONCONTEXTID;
+      throw OrthancException(ErrorCode_NetworkProtocol,
+                             "C-GET SCP: storeSCU: No presentation context for: (" +
+                             std::string(dcmSOPClassUIDToModality(sopClassUid.c_str(), "OT")) +
+                             ") " + sopClassUid);
     }
     else
     {
-      LOG(INFO) << "C-GET SCP selected transfer syntax " << GetTransferSyntaxUid(selectedSyntax)
-                << ", for source instance with SOP class " << sopClassUid
-                << " and transfer syntax " << GetTransferSyntaxUid(sourceSyntax);
+      CLOG(INFO, DICOM) << "C-GET SCP selected transfer syntax " << GetTransferSyntaxUid(selectedSyntax)
+                        << ", for source instance with SOP class " << sopClassUid
+                        << " and transfer syntax " << GetTransferSyntaxUid(sourceSyntax);
 
       // make sure that we can send images in this presentation context
       T_ASC_PresentationContext pc;
@@ -282,11 +260,12 @@ namespace Orthanc
           pc.acceptedRole != ASC_SC_ROLE_SCUSCP)
       {
         // the role is not appropriate
-        nFailed_++;
+        failedCount_++;
         AddFailedUIDInstance(sopInstanceUid);
-        LOG(ERROR) << "C-GET SCP: storeSCU: [No presentation context with requestor SCP role for: ("
-                   << dcmSOPClassUIDToModality(sopClassUid.c_str(), "OT") << ") " << sopClassUid;
-        return DIMSE_NOVALIDPRESENTATIONCONTEXTID;
+        throw OrthancException(ErrorCode_NetworkProtocol,
+                               "C-GET SCP: storeSCU: [No presentation context with requestor SCP role for: (" +
+                               std::string(dcmSOPClassUIDToModality(sopClassUid.c_str(), "OT")) +
+                               ") " + sopClassUid);
       }
     }
 
@@ -304,8 +283,8 @@ namespace Orthanc
     T_DIMSE_C_StoreRSP rsp;
     memset(&rsp, 0, sizeof(rsp));
 
-    LOG(INFO) << "Store SCU RQ: MsgID " << msgId << ", ("
-              << dcmSOPClassUIDToModality(sopClassUid.c_str(), "OT") << ")";
+    CLOG(INFO, DICOM) << "Store SCU RQ: MsgID " << msgId << ", ("
+                      << dcmSOPClassUIDToModality(sopClassUid.c_str(), "OT") << ")";
     
     T_DIMSE_DetectedCancelParameters cancelParameters;
     memset(&cancelParameters, 0, sizeof(cancelParameters));
@@ -320,7 +299,7 @@ namespace Orthanc
       DcmDataset *stDetailTmp = NULL;
       cond = DIMSE_storeUser(
         assoc, presId, &req, NULL /* imageFileName */, dicom->getDataset(),
-        GetSubOpProgressCallback, this /* callbackData */,
+        ProgressCallback, NULL /* callbackData */,
         (timeout_ > 0 ? DIMSE_NONBLOCKING : DIMSE_BLOCKING), timeout_,
         &rsp, &stDetailTmp, &cancelParameters);
       stDetail.reset(stDetailTmp);
@@ -341,7 +320,7 @@ namespace Orthanc
         cond = DIMSE_storeUser(
           assoc, presId, &req, NULL /* imageFileName */,
           transcoded.GetParsed().getDataset(),
-          GetSubOpProgressCallback, this /* callbackData */,
+          ProgressCallback, NULL /* callbackData */,
           (timeout_ > 0 ? DIMSE_NONBLOCKING : DIMSE_BLOCKING), timeout_,
           &rsp, &stDetailTmp, &cancelParameters);
         stDetail.reset(stDetailTmp);
@@ -349,72 +328,72 @@ namespace Orthanc
       else
       {
         // Cannot transcode
-        nFailed_++;
+        failedCount_++;
         AddFailedUIDInstance(sopInstanceUid);
-        LOG(ERROR) << "C-GET SCP: Cannot transcode " << sopClassUid
-                   << " from transfer syntax " << GetTransferSyntaxUid(sourceSyntax)
-                   << " to " << GetTransferSyntaxUid(selectedSyntax);
-        return DIMSE_NOVALIDPRESENTATIONCONTEXTID;
+        throw OrthancException(ErrorCode_NotImplemented,
+                               "C-GET SCP: Cannot transcode " + sopClassUid +
+                               " from transfer syntax " + GetTransferSyntaxUid(sourceSyntax) +
+                               " to " + GetTransferSyntaxUid(selectedSyntax));
       }      
     }
+
+    bool isContinue;
     
     if (cond.good())
     {
-      if (cancelParameters.cancelEncountered)
       {
-        if (origPresId_ == cancelParameters.presId &&
-            origMsgId_ == cancelParameters.req.MessageIDBeingRespondedTo)
-        {
-          getCancelled_ = OFTrue;
-        }
-        else
-        {
-          LOG(ERROR) << "C-GET SCP: Unexpected C-Cancel-RQ encountered: pid=" << (int)cancelParameters.presId
-                     << ", mid=" << (int)cancelParameters.req.MessageIDBeingRespondedTo;
-        }
+        OFString str;
+        CLOG(TRACE, DICOM) << "Received Store Response following a C-GET:" << std::endl
+                           << DIMSE_dumpMessage(str, rsp, DIMSE_INCOMING);
       }
       
-      if (rsp.DimseStatus == STATUS_Success)
+      if (cancelParameters.cancelEncountered)
+      {
+        CLOG(INFO, DICOM) << "C-GET SCP: Received C-Cancel RQ";
+        isContinue = false;
+      }
+      else if (rsp.DimseStatus == STATUS_Success)
       {
         // everything ok
-        nCompleted_++;
+        completedCount_++;
+        isContinue = true;
       }
       else if ((rsp.DimseStatus & 0xf000) == 0xb000)
       {
         // a warning status message
         warningCount_++;
-        LOG(ERROR) << "C-GET SCP: Store Warning: Response Status: "
-                   << DU_cstoreStatusString(rsp.DimseStatus);
+        CLOG(ERROR, DICOM) << "C-GET SCP: Store Warning: Response Status: "
+                           << DU_cstoreStatusString(rsp.DimseStatus);
+        isContinue = true;
       }
       else
       {
-        nFailed_++;
+        failedCount_++;
         AddFailedUIDInstance(sopInstanceUid);
         // print a status message
-        LOG(ERROR) << "C-GET SCP: Store Failed: Response Status: "
-                   << DU_cstoreStatusString(rsp.DimseStatus);
+        CLOG(ERROR, DICOM) << "C-GET SCP: Store Failed: Response Status: "
+                           << DU_cstoreStatusString(rsp.DimseStatus);
+        isContinue = true;
       }
     }
     else
     {
-      nFailed_++;
+      failedCount_++;
       AddFailedUIDInstance(sopInstanceUid);
       OFString temp_str;
-      LOG(ERROR) << "C-GET SCP: storeSCU: Store Request Failed: " << DimseCondition::dump(temp_str, cond);
+      CLOG(ERROR, DICOM) << "C-GET SCP: storeSCU: Store Request Failed: "
+                         << DimseCondition::dump(temp_str, cond);
+      isContinue = true;
     }
     
     if (stDetail.get() != NULL)
     {
-      // It is impossible to directly use the "<<" stream construct
-      // with "DcmObject::PrintHelper" using MSVC2008
-      std::stringstream s;
-      DcmObject::PrintHelper obj(*stDetail);
-      obj.dcmobj_.print(s);
-
-      LOG(INFO) << "  Status Detail: " << s.str();
+      std::stringstream s;  // DcmObject::PrintHelper cannot be used with VS2008
+      stDetail->print(s);
+      CLOG(INFO, DICOM) << "  Status Detail: " << s.str();
     }
     
-    return cond;
+    return isContinue;
   }
 
   bool OrthancGetRequestHandler::LookupIdentifiers(std::list<std::string>& publicIds,
@@ -469,15 +448,15 @@ namespace Orthanc
 
         if (tmp.empty())
         {
-          LOG(ERROR) << "C-GET: Cannot locate resource \"" << tokens[i]
-                     << "\" at the " << EnumerationToString(level) << " level";
+          CLOG(ERROR, DICOM) << "C-GET: Cannot locate resource \"" << tokens[i]
+                             << "\" at the " << EnumerationToString(level) << " level";
           return false;
         }
         else
         {
-          for (size_t i = 0; i < tmp.size(); i++)
+          for (size_t j = 0; j < tmp.size(); j++)
           {
-            publicIds.push_back(tmp[i]);
+            publicIds.push_back(tmp[j]);
           }
         }
       }
@@ -487,16 +466,16 @@ namespace Orthanc
   }
 
 
-    OrthancGetRequestHandler::OrthancGetRequestHandler(ServerContext& context) :
-      context_(context)
-    {
-      position_ = 0;
-      nRemaining_ = 0;
-      nCompleted_  = 0;
-      warningCount_ = 0;
-      nFailed_ = 0;
-      timeout_ = 0;
-    }
+  OrthancGetRequestHandler::OrthancGetRequestHandler(ServerContext& context) :
+    context_(context),
+    position_(0),
+    completedCount_ (0),
+    warningCount_(0),
+    failedCount_(0),
+    timeout_(0),
+    allowTranscoding_(false)
+  {
+  }
 
 
   bool OrthancGetRequestHandler::Handle(const DicomMap& input,
@@ -507,7 +486,7 @@ namespace Orthanc
   {
     MetricsRegistry::Timer timer(context_.GetMetricsRegistry(), "orthanc_get_scp_duration_ms");
 
-    LOG(WARNING) << "C-GET-SCU request received from AET \"" << originatorAet << "\"";
+    CLOG(INFO, DICOM) << "C-GET-SCU request received from AET \"" << originatorAet << "\"";
 
     {
       DicomArray query(input);
@@ -515,9 +494,9 @@ namespace Orthanc
       {
         if (!query.GetElement(i).GetValue().IsNull())
         {
-          LOG(INFO) << "  " << query.GetElement(i).GetTag()
-                    << "  " << FromDcmtkBridge::GetTagName(query.GetElement(i))
-                    << " = " << query.GetElement(i).GetValue().GetContent();
+          CLOG(INFO, DICOM) << "  (" << query.GetElement(i).GetTag().Format()
+                            << ")  " << FromDcmtkBridge::GetTagName(query.GetElement(i))
+                            << " = " << context_.GetDeidentifiedContent(query.GetElement(i));
         }
       }
     }
@@ -527,8 +506,14 @@ namespace Orthanc
      **/
 
     const DicomValue* levelTmp = input.TestAndGetValue(DICOM_TAG_QUERY_RETRIEVE_LEVEL);
+    if (levelTmp == NULL ||
+        levelTmp->IsNull() ||
+        levelTmp->IsBinary())
+    {
+      throw OrthancException(ErrorCode_BadRequest,
+                             "C-GET request without the tag 0008,0052 (QueryRetrieveLevel)");
+    }
 
-    assert(levelTmp != NULL);
     ResourceType level = StringToResourceType(levelTmp->GetContent().c_str());      
 
 
@@ -540,7 +525,7 @@ namespace Orthanc
 
     if (!LookupIdentifiers(publicIds, level, input))
     {
-      LOG(ERROR) << "Cannot determine what resources are requested by C-GET";
+      CLOG(ERROR, DICOM) << "Cannot determine what resources are requested by C-GET";
       return false; 
     }
 
@@ -550,14 +535,33 @@ namespace Orthanc
     
     {
       OrthancConfiguration::ReaderLock lock;
-      remote_ = lock.GetConfiguration().GetModalityUsingAet(originatorAet);
+
+      RemoteModalityParameters remote;
+
+      if (lock.GetConfiguration().LookupDicomModalityUsingAETitle(remote, originatorAet))
+      {
+        allowTranscoding_ = (context_.IsTranscodeDicomProtocol() &&
+                             remote.IsTranscodingAllowed());
+      }
+      else if (lock.GetConfiguration().GetBooleanParameter("DicomAlwaysAllowGet", false))
+      {
+        CLOG(INFO, DICOM) << "C-GET: Allowing SCU request from unknown modality with AET: " << originatorAet;
+        allowTranscoding_ = context_.IsTranscodeDicomProtocol();
+      }
+      else
+      {
+        // This should never happen, given the test at bottom of
+        // "OrthancApplicationEntityFilter::IsAllowedRequest()"
+        throw OrthancException(ErrorCode_InexistentItem,
+                               "C-GET: Rejecting SCU request from unknown modality with AET: " + originatorAet);
+      }
     }
 
     for (std::list<std::string>::const_iterator
            resource = publicIds.begin(); resource != publicIds.end(); ++resource)
     {
-      LOG(INFO) << "C-GET: Sending resource " << *resource
-                << " to modality \"" << originatorAet << "\"";
+      CLOG(INFO, DICOM) << "C-GET: Sending resource " << *resource
+                        << " to modality \"" << originatorAet << "\"";
       
       std::list<std::string> tmp;
       context_.GetIndex().GetChildInstances(tmp, *resource);
@@ -570,11 +574,9 @@ namespace Orthanc
     }
 
     failedUIDs_.clear();
-    getCancelled_ = OFFalse;
 
-    nRemaining_ = GetSubOperationCount();
-    nCompleted_ = 0;
-    nFailed_ = 0;
+    completedCount_ = 0;
+    failedCount_ = 0;
     warningCount_ = 0;
     timeout_ = timeout;
 
