@@ -2,24 +2,13 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2021 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation, either version 3 of the
  * License, or (at your option) any later version.
- *
- * In addition, as a special exception, the copyright holders of this
- * program give permission to link the code of its release with the
- * OpenSSL project's "OpenSSL" library (or with modified versions of it
- * that use the same license as the "OpenSSL" library), and distribute
- * the linked executables. You must obey the GNU General Public License
- * in all respects for all of the code used other than "OpenSSL". If you
- * modify file(s) with this exception, you may extend this exception to
- * your version of the file(s), but you are not obligated to do so. If
- * you do not wish to do so, delete this exception statement from your
- * version. If you delete this exception statement from all source files
- * in the program, then also delete it here.
  * 
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -42,10 +31,20 @@
 
 namespace Orthanc
 {
-  void MergeStudyJob::AddSourceSeriesInternal(const std::string& series)
+  static void RegisterSeries(std::map<std::string, std::string>& target,
+                             const std::string& series)
   {
     // Generate a target SeriesInstanceUID for this series
-    seriesUidMap_[series] = FromDcmtkBridge::GenerateUniqueIdentifier(ResourceType_Series);
+    if (target.find(series) == target.end())
+    {
+      target[series] = FromDcmtkBridge::GenerateUniqueIdentifier(ResourceType_Series);
+    }
+  }
+  
+
+  void MergeStudyJob::AddSourceSeriesInternal(const std::string& series)
+  {
+    RegisterSeries(seriesUidMap_, series);
 
     // Add all the instances of the series as to be processed
     std::list<std::string> instances;
@@ -147,13 +146,12 @@ namespace Orthanc
     // Fix since Orthanc 1.5.8: Assign new "SOPInstanceUID", as the instance has been modified
     modified->ReplacePlainString(DICOM_TAG_SOP_INSTANCE_UID, FromDcmtkBridge::GenerateUniqueIdentifier(ResourceType_Instance));
 
-    DicomInstanceToStore toStore;
-    toStore.SetOrigin(origin_);
-    toStore.SetParsedDicomFile(*modified);
+    std::unique_ptr<DicomInstanceToStore> toStore(DicomInstanceToStore::CreateFromParsedDicomFile(*modified));
+    toStore->SetOrigin(origin_);
 
     std::string modifiedInstance;
-    if (GetContext().Store(modifiedInstance, toStore,
-                       StoreInstanceMode_Default) != StoreStatus_Success)
+    ServerContext::StoreResult result = GetContext().Store(modifiedInstance, *toStore, StoreInstanceMode_Default);
+    if (result.GetStatus() != StoreStatus_Success)
     {
       LOG(ERROR) << "Error while storing a modified instance " << instance;
       return false;
@@ -241,7 +239,7 @@ namespace Orthanc
   }
 
 
-  void MergeStudyJob::AddSource(const std::string& studyOrSeries)
+  void MergeStudyJob::AddSource(const std::string& publicId)
   {
     ResourceType level;
     
@@ -249,28 +247,31 @@ namespace Orthanc
     {
       throw OrthancException(ErrorCode_BadSequenceOfCalls);
     }
-    else if (!GetContext().GetIndex().LookupResourceType(level, studyOrSeries))
+    else if (!GetContext().GetIndex().LookupResourceType(level, publicId))
     {
       throw OrthancException(ErrorCode_UnknownResource,
-                             "Cannot find this resource: " + studyOrSeries);
+                             "Cannot find this resource: " + publicId);
     }
     else
     {
       switch (level)
       {
         case ResourceType_Study:
-          AddSourceStudyInternal(studyOrSeries);
+          AddSourceStudyInternal(publicId);
           break;
           
         case ResourceType_Series:
-          AddSourceSeries(studyOrSeries);
+          AddSourceSeries(publicId);
+          break;
+          
+        case ResourceType_Instance:
+          AddSourceInstance(publicId);
           break;
           
         default:
           throw OrthancException(ErrorCode_UnknownResource,
-                                 "This resource is neither a study, nor a series: " +
-                                 studyOrSeries + " is a " +
-                                 std::string(EnumerationToString(level)));
+                                 "This resource is neither a study, nor a series, nor an instance: " +
+                                 publicId + " is a " + std::string(EnumerationToString(level)));
       }
     }    
   }
@@ -323,6 +324,34 @@ namespace Orthanc
   }
 
 
+  void MergeStudyJob::AddSourceInstance(const std::string& instance)
+  {
+    std::string parentStudy, parentSeries;
+
+    if (IsStarted())
+    {
+      throw OrthancException(ErrorCode_BadSequenceOfCalls);
+    }
+    else if (!GetContext().GetIndex().LookupParent(parentSeries, instance, ResourceType_Series) ||
+             !GetContext().GetIndex().LookupParent(parentStudy, parentSeries, ResourceType_Study))
+    {
+      throw OrthancException(ErrorCode_UnknownResource,
+                             "This resource is not an instance: " + instance);
+    }
+    else if (parentStudy == targetStudy_)
+    {
+      throw OrthancException(ErrorCode_UnknownResource,
+                             "Cannot merge instance " + instance +
+                             " into its parent study " + targetStudy_);
+    }
+    else
+    {
+      RegisterSeries(seriesUidMap_, parentSeries);
+      AddInstance(instance);
+    }    
+  }
+  
+
   void MergeStudyJob::GetPublicContent(Json::Value& value)
   {
     CleaningInstancesJob::GetPublicContent(value);
