@@ -2,24 +2,13 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation, either version 3 of the
  * License, or (at your option) any later version.
- *
- * In addition, as a special exception, the copyright holders of this
- * program give permission to link the code of its release with the
- * OpenSSL project's "OpenSSL" library (or with modified versions of it
- * that use the same license as the "OpenSSL" library), and distribute
- * the linked executables. You must obey the GNU General Public License
- * in all respects for all of the code used other than "OpenSSL". If you
- * modify file(s) with this exception, you may extend this exception to
- * your version of the file(s), but you are not obligated to do so. If
- * you do not wish to do so, delete this exception statement from your
- * version. If you delete this exception statement from all source files
- * in the program, then also delete it here.
  * 
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -74,7 +63,7 @@ namespace Orthanc
 
 
   void SystemCallOperation::Apply(JobOperationValues& outputs,
-                                  const JobOperationValue& input)
+                                  const IJobOperationValue& input)
   {
     std::vector<std::string> arguments = preArguments_;
 
@@ -84,7 +73,7 @@ namespace Orthanc
     
     switch (input.GetType())
     {
-      case JobOperationValue::Type_DicomInstance:
+      case IJobOperationValue::Type_DicomInstance:
       {
         const DicomInstanceOperationValue& instance =
           dynamic_cast<const DicomInstanceOperationValue&>(input);
@@ -103,7 +92,7 @@ namespace Orthanc
         break;
       }
 
-      case JobOperationValue::Type_String:
+      case IJobOperationValue::Type_String:
       {
         const StringOperationValue& value =
           dynamic_cast<const StringOperationValue&>(input);
@@ -112,7 +101,7 @@ namespace Orthanc
         break;
       }
 
-      case JobOperationValue::Type_Null:
+      case IJobOperationValue::Type_Null:
         break;
 
       default:
