@@ -2,24 +2,13 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2021 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation, either version 3 of the
  * License, or (at your option) any later version.
- *
- * In addition, as a special exception, the copyright holders of this
- * program give permission to link the code of its release with the
- * OpenSSL project's "OpenSSL" library (or with modified versions of it
- * that use the same license as the "OpenSSL" library), and distribute
- * the linked executables. You must obey the GNU General Public License
- * in all respects for all of the code used other than "OpenSSL". If you
- * modify file(s) with this exception, you may extend this exception to
- * your version of the file(s), but you are not obligated to do so. If
- * you do not wish to do so, delete this exception statement from your
- * version. If you delete this exception statement from all source files
- * in the program, then also delete it here.
  * 
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -69,6 +58,7 @@ static const char* const KEY_DICOM_TLS_ENABLED = "DicomTlsEnabled";
 static const char* const KEY_DICOM_TLS_CERTIFICATE = "DicomTlsCertificate";
 static const char* const KEY_DICOM_TLS_TRUSTED_CERTIFICATES = "DicomTlsTrustedCertificates";
 static const char* const KEY_MAXIMUM_PDU_LENGTH = "MaximumPduLength";
+static const char* const KEY_DICOM_TLS_REMOTE_CERTIFICATE_REQUIRED = "DicomTlsRemoteCertificateRequired";
 
 
 class OrthancStoreRequestHandler : public IStoreRequestHandler
@@ -83,10 +73,10 @@ public:
   }
 
 
-  virtual void Handle(DcmDataset& dicom,
-                      const std::string& remoteIp,
-                      const std::string& remoteAet,
-                      const std::string& calledAet) ORTHANC_OVERRIDE 
+  virtual uint16_t Handle(DcmDataset& dicom,
+                          const std::string& remoteIp,
+                          const std::string& remoteAet,
+                          const std::string& calledAet) ORTHANC_OVERRIDE 
   {
     std::unique_ptr<DicomInstanceToStore> toStore(DicomInstanceToStore::CreateFromDcmDataset(dicom));
     
@@ -96,8 +86,11 @@ public:
                          (remoteIp.c_str(), remoteAet.c_str(), calledAet.c_str()));
 
       std::string id;
-      context_.Store(id, *toStore, StoreInstanceMode_Default);
+      ServerContext::StoreResult result = context_.Store(id, *toStore, StoreInstanceMode_Default);
+      return result.GetCStoreStatusCode();
     }
+
+    return STATUS_STORE_Error_CannotUnderstand;
   }
 };
 
@@ -277,7 +270,9 @@ private:
   ServerContext&  context_;
   bool            alwaysAllowEcho_;
   bool            alwaysAllowFind_;  // New in Orthanc 1.9.0
+  bool            alwaysAllowFindWorklist_; // New in Orthanc 1.10.0
   bool            alwaysAllowGet_;   // New in Orthanc 1.9.0
+  bool            alwaysAllowMove_;  // New in Orthanc 1.9.7
   bool            alwaysAllowStore_;
 
 public:
@@ -288,7 +283,9 @@ public:
       OrthancConfiguration::ReaderLock lock;
       alwaysAllowEcho_ = lock.GetConfiguration().GetBooleanParameter("DicomAlwaysAllowEcho", true);
       alwaysAllowFind_ = lock.GetConfiguration().GetBooleanParameter("DicomAlwaysAllowFind", false);
+      alwaysAllowFindWorklist_ = lock.GetConfiguration().GetBooleanParameter("DicomAlwaysAllowFindWorklist", false);
       alwaysAllowGet_ = lock.GetConfiguration().GetBooleanParameter("DicomAlwaysAllowGet", false);
+      alwaysAllowMove_ = lock.GetConfiguration().GetBooleanParameter("DicomAlwaysAllowMove", false);
       alwaysAllowStore_ = lock.GetConfiguration().GetBooleanParameter("DicomAlwaysAllowStore", true);
     }
 
@@ -297,10 +294,20 @@ public:
       LOG(WARNING) << "Security risk in DICOM SCP: C-FIND requests are always allowed, even from unknown modalities";
     }
 
+    if (alwaysAllowFindWorklist_)
+    {
+      LOG(WARNING) << "Security risk in DICOM SCP: C-FIND requests for worklists are always allowed, even from unknown modalities";
+    }
+
     if (alwaysAllowGet_)
     {
       LOG(WARNING) << "Security risk in DICOM SCP: C-GET requests are always allowed, even from unknown modalities";
     }
+
+    if (alwaysAllowMove_)
+    {
+      LOG(WARNING) << "Security risk in DICOM SCP: C-MOOVE requests are always allowed, even from unknown modalities";
+    }
   }
 
   virtual bool IsAllowedConnection(const std::string& remoteIp,
@@ -312,7 +319,9 @@ public:
 
     if (alwaysAllowEcho_ ||
         alwaysAllowFind_ ||
+        alwaysAllowFindWorklist_ ||
         alwaysAllowGet_ ||
+        alwaysAllowMove_ ||
         alwaysAllowStore_)
     {
       return true;
@@ -324,6 +333,17 @@ public:
     }
   }
 
+  static void ReportDisallowedCommand(const std::string& remoteIp,
+                                      const std::string& remoteAet,
+                                      DicomRequestType type)
+  {
+    LOG(WARNING) << "DICOM authorization rejected for AET " << remoteAet
+                 << " on IP " << remoteIp << ": The DICOM command "
+                 << EnumerationToString(type) << " is not allowed for this modality "
+                 << "according to configuration option \"DicomModalities\"";
+  }
+  
+
   virtual bool IsAllowedRequest(const std::string& remoteIp,
                                 const std::string& remoteAet,
                                 const std::string& calledAet,
@@ -344,6 +364,12 @@ public:
       // Incoming C-Find requests are always accepted, even from unknown AET
       return true;
     }
+    else if (type == DicomRequestType_FindWorklist &&
+             alwaysAllowFindWorklist_)
+    {
+      // Incoming C-Find requests for worklists are always accepted, even from unknown AET
+      return true;
+    }
     else if (type == DicomRequestType_Store &&
              alwaysAllowStore_)
     {
@@ -356,35 +382,76 @@ public:
       // Incoming C-Get requests are always accepted, even from unknown AET
       return true;
     }
+    else if (type == DicomRequestType_Move &&
+             alwaysAllowMove_)
+    {
+      // Incoming C-Move requests are always accepted, even from unknown AET
+      return true;
+    }
     else
     {
-      OrthancConfiguration::ReaderLock lock;
-
+      bool checkIp;
       std::list<RemoteModalityParameters> modalities;
-      if (lock.GetConfiguration().LookupDicomModalitiesUsingAETitle(modalities, remoteAet))
-      {
-        if (modalities.size() == 1) // don't check the IP if there's only one modality with this AET
-        {
-          return modalities.front().IsRequestAllowed(type);
-        }
-        else // if there are multiple modalities with the same AET, check the one matching this IP
-        {
-          for (std::list<RemoteModalityParameters>::const_iterator it = modalities.begin(); it != modalities.end(); ++it)
-          {
-            if (it->GetHost() == remoteIp)
-            {
-              return it->IsRequestAllowed(type);
-            }
-          }
 
-          LOG(WARNING) << "Unable to check DICOM authorization for AET " << remoteAet
-                       << " on IP " << remoteIp << ", " << modalities.size()
-                       << " modalites found with this AET but none of them matching the IP";
-        }
+      {
+        OrthancConfiguration::ReaderLock lock;
+        lock.GetConfiguration().LookupDicomModalitiesUsingAETitle(modalities, remoteAet);
+        checkIp = lock.GetConfiguration().GetBooleanParameter("DicomCheckModalityHost", false);
+      }
+      
+      if (modalities.empty())
+      {
+        LOG(WARNING) << "DICOM authorization rejected  for AET " << remoteAet
+                     << " on IP " << remoteIp << ": This AET is not listed in "
+                     << "configuration option \"DicomModalities\"";
         return false;
       }
+      else if (modalities.size() == 1)
+      {
+        // DicomCheckModalityHost is true: check if the IP match the configured IP
+        if (checkIp &&
+            remoteIp != modalities.front().GetHost())
+        {
+          LOG(WARNING) << "DICOM authorization rejected for AET " << remoteAet
+                       << " on IP " << remoteIp << ": Its IP address should be "
+                       << modalities.front().GetHost()
+                       << " according to configuration option \"DicomModalities\"";
+          return false;
+        }
+        else if (modalities.front().IsRequestAllowed(type))
+        {
+          return true;
+        }
+        else
+        {
+          ReportDisallowedCommand(remoteIp, remoteAet, type);
+          return false;
+        }
+      }
       else
       {
+        // If there are multiple modalities with the same AET, consider the one matching this IP
+        for (std::list<RemoteModalityParameters>::const_iterator
+               it = modalities.begin(); it != modalities.end(); ++it)
+        {
+          if (it->GetHost() == remoteIp)
+          {
+            if (it->IsRequestAllowed(type))
+            {
+              return true;
+            }
+            else
+            {
+              ReportDisallowedCommand(remoteIp, remoteAet, type);
+              return false;
+            }
+          }
+        }
+
+        LOG(WARNING) << "DICOM authorization rejected for AET " << remoteAet
+                     << " on IP " << remoteIp << ": " << modalities.size()
+                     << " modalites found with this AET in configuration option "
+                     << "\"DicomModalities\", but none of them matches the IP";
         return false;
       }
     }
@@ -650,8 +717,9 @@ static void PrintVersion(const char* path)
   std::cout
     << path << " " << ORTHANC_VERSION << std::endl
     << "Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics Department, University Hospital of Liege (Belgium)" << std::endl
-    << "Copyright (C) 2017-2021 Osimis S.A. (Belgium)" << std::endl
-    << "Licensing GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>, with OpenSSL exception." << std::endl
+    << "Copyright (C) 2017-2022 Osimis S.A. (Belgium)" << std::endl
+    << "Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain (Belgium)" << std::endl
+    << "Licensing GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>." << std::endl
     << "This is free software: you are free to change and redistribute it." << std::endl
     << "There is NO WARRANTY, to the extent permitted by law." << std::endl
     << std::endl
@@ -1063,6 +1131,8 @@ static bool StartHttpServer(ServerContext& context,
         httpServer.SetSslVerifyPeers(false);
       }
 
+      LOG(INFO) << "Version of Lua: " << LUA_VERSION;
+
       if (lock.GetConfiguration().GetBooleanParameter("ExecuteLuaEnabled", false))
       {
         context.SetExecuteLuaEnabled(true);
@@ -1147,6 +1217,7 @@ static bool StartDicomServer(ServerContext& context,
       dicomServer.SetCalledApplicationEntityTitleCheck(lock.GetConfiguration().GetBooleanParameter("DicomCheckCalledAet", false));
       dicomServer.SetAssociationTimeout(lock.GetConfiguration().GetUnsignedIntegerParameter("DicomScpTimeout", 30));
       dicomServer.SetPortNumber(lock.GetConfiguration().GetUnsignedIntegerParameter("DicomPort", 4242));
+      dicomServer.SetThreadsCount(lock.GetConfiguration().GetUnsignedIntegerParameter("DicomThreadsCount", 4));
       dicomServer.SetApplicationEntityTitle(lock.GetConfiguration().GetOrthancAET());
 
       // Configuration of DICOM TLS for Orthanc SCP (since Orthanc 1.9.0)
@@ -1161,6 +1232,10 @@ static bool StartDicomServer(ServerContext& context,
       }
 
       dicomServer.SetMaximumPduLength(lock.GetConfiguration().GetUnsignedIntegerParameter(KEY_MAXIMUM_PDU_LENGTH, 16384));
+
+      // New option in Orthanc 1.9.3
+      dicomServer.SetRemoteCertificateRequired(
+        lock.GetConfiguration().GetBooleanParameter(KEY_DICOM_TLS_REMOTE_CERTIFICATE_REQUIRED, true));
     }
 
 #if ORTHANC_ENABLE_PLUGINS == 1
@@ -1419,6 +1494,10 @@ static bool ConfigureServerContext(IDatabaseWrapper& database,
       lock.GetConfiguration().GetStringParameter(KEY_DICOM_TLS_TRUSTED_CERTIFICATES, ""));
     DicomAssociationParameters::SetDefaultMaximumPduLength(
       lock.GetConfiguration().GetUnsignedIntegerParameter(KEY_MAXIMUM_PDU_LENGTH, 16384));
+
+    // New option in Orthanc 1.9.3
+    DicomAssociationParameters::SetDefaultRemoteCertificateRequired(
+      lock.GetConfiguration().GetBooleanParameter(KEY_DICOM_TLS_REMOTE_CERTIFICATE_REQUIRED, true));
   }
   
   ServerContext context(database, storageArea, false /* not running unit tests */, maxCompletedJobs);
@@ -1450,6 +1529,16 @@ static bool ConfigureServerContext(IDatabaseWrapper& database,
     {
       context.GetIndex().SetMaximumStorageSize(0);
     }
+
+    try
+    {
+      uint64_t size = lock.GetConfiguration().GetUnsignedIntegerParameter("MaximumStorageCacheSize", 128);
+      context.SetMaximumStorageCacheSize(size * 1024 * 1024);
+    }
+    catch (...)
+    {
+      context.SetMaximumStorageCacheSize(128);
+    }
   }
 
   {
@@ -1493,6 +1582,7 @@ static bool ConfigureDatabase(IDatabaseWrapper& database,
     static const char* const CHECK_REVISIONS = "CheckRevisions";
     
     OrthancConfiguration::ReaderLock lock;
+    
     if (lock.GetConfiguration().GetBooleanParameter(CHECK_REVISIONS, false))
     {
       if (database.HasRevisionsSupport())
@@ -1505,6 +1595,16 @@ static bool ConfigureDatabase(IDatabaseWrapper& database,
         LOG(WARNING) << "The custom database back-end has *no* support for revisions of metadata and attachments, "
                      << "but configuration option \"" << CHECK_REVISIONS << "\" is set to \"true\"";
       }
+      
+      static const char* const STORE_MD5 = "StoreMD5ForAttachments";
+
+      if (!lock.GetConfiguration().GetBooleanParameter(STORE_MD5, true))
+      {
+        throw OrthancException(
+          ErrorCode_ParameterOutOfRange,
+          "The revision system is enabled by configuration option \"" + std::string(CHECK_REVISIONS) +
+          "\", but won't work properly for attachments if \"" + std::string(STORE_MD5) + "\" is set to \"false\"");
+      }
     }
   }
 
